/* HickPrj 1.0.2 -- Login project chooser for hick.
** Copyright (C) 1999 - 2001 Daniel Bergstroem <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>

#include <gtk/gtkrc.h>
#include <gtk/gtkmain.h>

#include "hickpc.h"
#include "chooser.h"

static int
usage(const char *filename)
{
  printf("Usage: %s [ OPTIONS ] PROJECTS\n\n"
         "\t-h\t\tPrint this help test.\n"
         "\t-v\t\tShow version information.\n"
         "\t-d FILE\t\tFile containing user description text.\n"
         "\t-T\t\tTitle of the chooser.\n"
         "\t-t SECONDS\tTimeout before auto-exit.\n"
         "\t-P STRING\tOther user hick home then ~/.hick.\n\n"
         "Project format:\n\n"
         "  NAME ':' DATA ':' HOME [ ':' NAME ':' DATA ':' HOME ... ]\n\n"
         "\tNAME\t\tname on the project.\n"
         "\tDATA\t\tpath to project data files.\n"
         "\tHOME\t\thome directory of project.\n\n", filename);
  return 0;
}


int
main(int argc, char **argv)
{
  int n;
  int t = 0;
  Projects prjs;
  struct passwd *pwd;
  char *header = PROGRAM, *desc = 0, *path = 0, pwdpath[FNAME_LEN + 1];

  gtk_init(&argc, &argv);
  gtk_rc_init();

  while((n = getopt(argc, argv, "vhT:t:d:P:")) != EOF) {
    switch(n) {
      case 'T': { /* Title */
        header = optarg;
        break;
      }
      case 't': { /* timeout */
        t = atoi(optarg);
        break;
      }
      case 'd': { /* user Description */
        desc = optarg;
        break;
      }
      case 'P': { /* change user path */
        path = optarg;
        break;
      }
      case 'v': {
        printf("hickprj %s\n", VERSION_STRING);
        return 0;
      }
      case 'h': {
        return usage(argv[0]);
        break;
      }
      default: {
        printf("%s: unknown option.\n", argv[0]);
        usage(argv[0]);
        return 1;
      }
    }
  }

  if(optind == argc) {
    printf("Error: missing project information.\n");
    return 1;
  }

  if(!(pwd = getpwuid(geteuid()))) {
    return 1;
  }

  if(!path) {
    hsh_create_str(pwdpath, FNAME_LEN, pwd->pw_dir, "/.hick", NULL);
    path = pwdpath;
  }

  projects_init(&prjs, argv[optind], pwd->pw_name, desc, path);

  if(prjs.projects.siz <= 1) {
    return 1;
  }

  gtk_widget_show_all(chooser_new(&prjs, t, header, pwd->pw_dir));

  gtk_main();
  return 0;
}
