/* HickUP 1.2 -- Graphical interface to hick.
** Copyright (C) 1998 - 2001, 2008 Daniel Bergstrom <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtkmain.h>
#include <gtk/gtknotebook.h>

#include "gui.h"
#include "filter.h"
#include "update.h"
#include "buttons.h"

static void cb_ok(GtkWidget *w, Project *prj);
static void cb_edit_ok(GtkWidget *w, Project *prj);

static void cb_filter_apply(GtkWidget *w, Project *prj);
static void cb_filter_cancel(GtkWidget *w, Project *prj);

static int _ok_button = FALSE;


void
buttons_signal(GUI_Main *gmain, guint page_num, Project *prj)
{
  if(!gmain->bok) {
    return;
  }

  if(gmain->sok) {
    gtk_signal_disconnect(GTK_OBJECT(gmain->bok), gmain->sok);
    gtk_signal_disconnect(GTK_OBJECT(gmain->bapply), gmain->sapply);
    gtk_signal_disconnect(GTK_OBJECT(gmain->bcancel), gmain->scancel);
  }

  switch(page_num) {
    case 0: {
      gmain->sok = gtk_signal_connect(GTK_OBJECT(gmain->bok), "clicked", 
                                      GTK_SIGNAL_FUNC(cb_ok), prj);

      gmain->sapply = gtk_signal_connect(GTK_OBJECT(gmain->bapply), "clicked", 
                                         GTK_SIGNAL_FUNC(cb_apply), prj);

      gmain->scancel = gtk_signal_connect(GTK_OBJECT(gmain->bcancel),"clicked",
                                          GTK_SIGNAL_FUNC(cb_quit), NULL);
      break;
    }
    case 1: {
      gmain->sok = gtk_signal_connect(GTK_OBJECT(gmain->bok), "clicked", 
                                      GTK_SIGNAL_FUNC(cb_filter_apply), prj);

      gmain->sapply = gtk_signal_connect(GTK_OBJECT(gmain->bapply), "clicked", 
                                         GTK_SIGNAL_FUNC(cb_filter_apply),prj);

      gmain->scancel = gtk_signal_connect(GTK_OBJECT(gmain->bcancel),"clicked",
                                          GTK_SIGNAL_FUNC(cb_filter_cancel),
                                          prj);
      break;
    }
    case 2: {
      gmain->sok = gtk_signal_connect(GTK_OBJECT(gmain->bok), "clicked", 
                                      GTK_SIGNAL_FUNC(cb_edit_ok), prj);

      gmain->sapply = gtk_signal_connect(GTK_OBJECT(gmain->bapply), "clicked", 
                                         GTK_SIGNAL_FUNC(cb_edit_apply), prj);

      gmain->scancel = gtk_signal_connect(GTK_OBJECT(gmain->bcancel),"clicked",
                                          GTK_SIGNAL_FUNC(cb_quit), NULL);
      break;
    }
  }
}


void
cb_quit(GtkWidget *w, gpointer data)
{
  if(update_is_running()) {
    return;
  }

  gtk_main_quit();
}


static void
cb_ok(GtkWidget *w, Project *prj)
{
  gtk_widget_grab_focus(_gmain.bok);

  if(update_is_running()) {
    return;
  }

  _ok_button = TRUE;
  cb_apply(NULL, prj);
}


void
cb_apply(GtkWidget *w, Project *prj)
{
  if(update_is_running()) {
    gtk_widget_grab_focus(_gmain.bok);
    return;
  }

  reqchk_close(&prj->packets.reqchk);
  reqchk_check(&prj->packets.reqchk, FALSE);

  if(!reqchk_show(&prj->packets.reqchk)) {
    cb_update(w, prj);
  }

  gtk_widget_grab_focus(_gmain.bok);
}


void
cb_update(GtkWidget *w, Project *prj)
{
  static Progress pbar = { { NULL, NULL }, 0, 0, NULL, NULL, NULL };

  update_set_running(TRUE);
  reqchk_close(&prj->packets.reqchk);

  update_progress_bar_new("Update", 0, &pbar);

  update_run(prj, &_gmain.projects, &pbar);
  update_run_mk(prj, &_gmain.projects, &pbar);

  gtk_widget_hide(pbar.dialog.window);
  update_set_running(FALSE);

  if(!update_show_report(prj) && _ok_button) {
    cb_quit(w, prj); /* OK button where pressed, lets quit.. */
  }

  select_clear(&prj->packets);
  hview_freeze(&prj->packets.hview);
  hview_display_destroy(&prj->packets.hview);
  avail_free(&prj->packets.avail);

  prj->packets.has_init = 0;

  if(gtk_notebook_get_current_page(GTK_NOTEBOOK(prj->notebook)) == 0) {
    project_init_packets(prj, &_gmain.projects.packets, 
                         &_gmain.projects.types);
  }
  hview_thaw(&prj->packets.hview);

  gtk_widget_destroy(pbar.dialog.window);
}


void
update_all(Projects *prjs, int wontexit)
{
  size_t n;
  static Progress pbar = { { NULL, NULL }, 0, 0, NULL, NULL, NULL };

  update_set_running(TRUE);

  update_progress_bar_new("Update", 0, &pbar);

  for(n = 0; n < prjs->projects.siz; n++) {
    prjs->current = parr_get(&prjs->projects, Project, n);
    reqchk_close(&prjs->current->packets.reqchk);

    if(!prjs->current->packets.has_init) {
      project_init_packets(prjs->current, &_gmain.projects.packets, 
                           &_gmain.projects.types);
    }
    
    update_run(prjs->current, &_gmain.projects, &pbar);
    update_run_mk(prjs->current, &_gmain.projects, &pbar);
  }

  if(!wontexit) {
    return;
  }

  cb_reload_projects(0, 0);
  prjs->current = parr_get(&prjs->projects, Project, n);

  gtk_widget_destroy(pbar.dialog.window);
  update_set_running(FALSE);
}


void
update_and_exit(Projects *prjs, int level, const char *prj)
{
  size_t n;
  static Progress pbar = { { NULL, NULL }, 0, 0, NULL, NULL, NULL };

  switch(level) {
    case 1: { /* update user */
      update_progress_bar_new("Update", 0, &pbar);

      if(!prjs->current->packets.has_init) {
        project_init_packets(prjs->current, &_gmain.projects.packets, 
                             &_gmain.projects.types);
      }

      update_run(prjs->current, &_gmain.projects, &pbar);
      update_run_mk(prjs->current, &_gmain.projects, &pbar);
      break;
    }
    case 2: { /* update specified project */
      for(n = 0; n < prjs->projects.siz; n++) {
        if(strcmp(prj, parr_get(&prjs->projects, Project, n)->name)) {
          continue;
        }

        prjs->current = parr_get(&prjs->projects, Project, n);

        if(!prjs->current->packets.has_init) {
          project_init_packets(prjs->current, &_gmain.projects.packets, 
                               &_gmain.projects.types);
        }

        update_progress_bar_new("Update", 0, &pbar);
        update_run(prjs->current, &_gmain.projects, &pbar);
        update_run_mk(prjs->current, &_gmain.projects, &pbar);
        exit(0);
      }

      printf("Couldn't find project '%s'\n", prj);
      break;
    }
    case 3: { /* update all projects */
      update_all(prjs, 0);
      break;
    }
  }

  exit(0);
}


static void
cb_edit_ok(GtkWidget *w, Project *prj)
{
  _ok_button = TRUE;
  cb_edit_apply(NULL, prj);
  gtk_widget_grab_focus(_gmain.bok);
}


void
cb_edit_apply(GtkWidget *w, Project *prj)
{
  reqchk_close(&prj->edit->reqchk);
  reqchk_check(&prj->edit->reqchk, TRUE);

  if(!reqchk_show(&prj->edit->reqchk)) {
    cb_edit_save(w, prj);
  }

  gtk_widget_grab_focus(_gmain.bok);
}


void
cb_edit_save(GtkWidget *w, Project *prj)
{
  SpecVars svars;

  reqchk_close(&prj->edit->reqchk);
  reqchk_close(&prj->packets.reqchk);

  spec_vars_init(&svars);

  avail_to_specvars(&prj->edit->avail, &_gmain.projects.types, &svars, TRUE);

  spec_save(prj->specfile, &svars);
  spec_free(&svars);

  if(_ok_button) { /* OK button pressed, lets quit */
    cb_quit(w, prj);
  }

  select_clear(&prj->packets);
  hview_freeze(&prj->packets.hview);
  hview_display_destroy(&prj->packets.hview);
  avail_free(&prj->packets.avail);

  prj->packets.has_init = 0;

  if(gtk_notebook_get_current_page(GTK_NOTEBOOK(prj->notebook)) == 0) {
    project_init_packets(prj, &_gmain.projects.packets, 
                         &_gmain.projects.types);
  }
  hview_thaw(&prj->packets.hview);
}


void
cb_reqchk_cancel(GtkWidget *w, ReqChk *rchk)
{
  reqchk_close(rchk);
  _ok_button = FALSE; /* changed our minds? */
}


static void
cb_filter_apply(GtkWidget *w, Project *prj)
{
  TabPackets *tab = &prj->packets;

  filter_gui_get(&prj->filter.filter, &prj->filter);
  filter_copy(&tab->filter, &prj->filter.filter);

  select_clear(tab);
  hview_freeze(&tab->hview);
  hview_display_destroy(&tab->hview);
  avail_free(&tab->avail);
  hview_thaw(&tab->hview);

  tab->has_init = 0;
  gtk_widget_grab_focus(_gmain.bok);
}


static void
cb_filter_cancel(GtkWidget *w, Project *prj)
{
  filter_gui_set(&prj->filter.filter, &prj->filter);
  gtk_widget_grab_focus(_gmain.bok);
}
