/* HickUP 1.2 -- Graphical interface to hick.
** Copyright (C) 1998 - 2001, 2008 Daniel Bergstrom <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "gpixmap.h"

#include "no.xpm"
#include "yes.xpm"
#include "dot.xpm"
#include "empty.xpm"
#include "forced.xpm"
#include "select.xpm"
#include "default.xpm"
#include "selected.xpm"
#include "attention.xpm"
#include "folder_open.xpm"
#include "folder_closed.xpm"
#include "packet_open.xpm"
#include "packet_closed.xpm"

static void create_pixmap(const char **parr, GtkWidget *window, GPixmap *res);


void
gui_pixmap_init(GUI_Pixmaps *guip)
{
  gpixmap_init(&guip->no);
  gpixmap_init(&guip->yes);

  gpixmap_init(&guip->dot);
  gpixmap_init(&guip->empty);

  gpixmap_init(&guip->select);
  gpixmap_init(&guip->dselected);
  gpixmap_init(&guip->forced);
  gpixmap_init(&guip->selected);
  gpixmap_init(&guip->attention);

  dgpixmap_init(&guip->group);
  dgpixmap_init(&guip->packet);

  guip->pixmaps[0] = guip->pixmaps[1] = guip->pixmaps[2] = NULL;
  guip->pixmaps[3] = NULL;
}


void
gui_pixmap_create_dot(GUI_Pixmaps *guip, GtkWidget *window)
{
  if(!guip->dot.pix && !guip->dot.mask) {
    gtk_widget_realize(window);
    create_pixmap(dot_xpm, window, &guip->dot);
  }
}


void
gui_pixmap_create_all(GUI_Pixmaps *guip, GtkWidget *window)
{
  gtk_widget_realize(window);

  create_pixmap(no_xpm, window, &guip->no);
  create_pixmap(yes_xpm, window, &guip->yes);

  gui_pixmap_create_dot(guip, window);
  create_pixmap(empty_xpm, window, &guip->empty);

  create_pixmap(select_xpm, window, &guip->select);
  create_pixmap(default_xpm, window, &guip->dselected);
  create_pixmap(forced_xpm, window, &guip->forced);
  create_pixmap(selected_xpm, window, &guip->selected);
  create_pixmap(attention_xpm, window, &guip->attention);

  create_pixmap(folder_open_xpm, window, &guip->group.open);
  create_pixmap(folder_closed_xpm, window, &guip->group.close);
  create_pixmap(packet_open_xpm, window, &guip->packet.open);
  create_pixmap(packet_closed_xpm, window, &guip->packet.close);

  guip->pixmaps[0] = &guip->group;  /* type    */
  guip->pixmaps[1] = &guip->group;  /* group   */
  guip->pixmaps[2] = &guip->packet; /* packet  */
  guip->pixmaps[3] = NULL;          /* version */
}


static void
create_pixmap(const char **parr, GtkWidget *window, GPixmap *res)
{
  res->pix = gdk_pixmap_create_from_xpm_d(window->window, &res->mask, 0,
                                          (gchar **)parr);
}
