/* HickUP 1.2 -- Graphical interface to hick.
** Copyright (C) 1998 - 2001, 2008 Daniel Bergstrom <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _HVIEW_H
#define _HVIEW_H

typedef struct _hup_hview HView;

#include <gtk/gtkctree.h>
#include <gtk/gtkclist.h>

#include "node.h"
#include "array.h"
#include "gpixmap.h"

/* getting a bit to nested with all the includes here  : ( */

#ifndef _AVAIL_H
typedef struct _hup_availpacket AvailPacket; /* from avail.h */
typedef struct _hup_availversion AvailVersion;
#endif /* _AVAIL_H */

#ifndef _REFS_H
typedef struct _hup_nodetree NodeTree; /* from refs.h */
#endif /* _REFS_H */

#ifndef _PROJECT_H
typedef struct _hup_project Project; /* from project.h */
#endif /* _PROJECT_H */

typedef GPixmap *(*HViewPixmapFunc)(AvailPacket *apack, AvailVersion *aver, 
                                    void *data);

enum hview_type { HVIEW_CTREE, HVIEW_CLIST };

enum hview_menu_type {
  MENU_ALL, /* special, not a valid type for sensitive */
  MENU_VIEW,
  MENU_COPY_PRJ,
  MENU_ENABLE,
  MENU_DISABLE,
  MENU_FORCE,
  MENU_ENABLE_PACKET,
  MENU_DISABLE_PACKET,
  MENU_DEFAULT
};

/* HViewMenuItem *********************************************************** */

typedef struct {
  enum hview_menu_type type;
  GtkWidget *item;
  GtkWidget *pixmap;
  GtkWidget *label;
} HViewMenuItem;

/* HViewMenu *************************************************************** */

typedef struct {
  GtkWidget *menu;
  PArr items; /* (HViewMenuItem *) */
} HViewMenu;

/* HViewSelect ************************************************************* */

typedef struct {
  NodeTree *ntree;
  AvailPacket *packet;
  AvailVersion *version;
} HViewSelect;

/* HView ******************************************************************* */

struct _hup_hview {
  HViewMenu menu;
  HViewSelect select;

  GtkWidget *view;      /* can be a CTree or CList */
  enum hview_type type; /* what the type it is */

  Project *project;
  NodeList *avail;      /* pointer to list of avail packets info. */

  NodeList display;     /* List of displaied Refs (refs.h) */

  HViewPixmapFunc pixfunc; /* function to get pixmap */
};

#include "avail.h"

#ifdef __cplusplus
extern "C" {
#endif

extern GtkWidget *hview_new(HView *hview, enum hview_type type, gint columns,
                            gchar **titles, gint *cwidth, Project *prj);

/* char *name, GPixmap *pixmap, enum hview_menu_type type, GtkSignalFunc */
#if __GNUC__ >= 4
extern void hview_menu_create(HView *hview, ...)
  __attribute__((__sentinel__));
#else
extern void hview_menu_create(HView *hview, ...);
#endif

extern void hview_menu_set_sensitive(HView *hview, enum hview_menu_type type, 
                                     gboolean sensitive);
extern void hview_menu_set_show(HView *hview, enum hview_menu_type type, 
                                gboolean show);
extern void hview_menu_set_label(HView *hview, enum hview_menu_type type, 
                                 const char *p);
extern void hview_menu_set_gpixmap(HView *hview, enum hview_menu_type type, 
                                   GPixmap *pixi);

extern void hview_display_clear(HView *hview);
extern void hview_display_destroy(HView *hview);
#define hview_display_get(hview) (&(hview)->display)
#define hview_display_set(hview, list) \
  ((view)->display.head = (list)->head, ((view)->display.tail = (list)->tail

extern int hview_select_set(HView *hview, NodeTree *ntree);
extern int hview_selected_get_avail(HView  *hview, AvailPacket **apack, 
                                    AvailVersion **aver);
extern int hview_select_get_avail(HView *hview, GdkEventButton *event, 
                                  AvailPacket **apack, AvailVersion **aver,
                                  gint *row, gint *column);
extern void hview_select_row(HView *hview, gint row);
#define hview_select_clear(hview) \
   (hview)->select.packet = NULL; (hview)->select.version = NULL; \
   (hview)->select.ntree = NULL

extern void hview_pixmap_display(HView *hview, Node *node, GPixmap *pixi);
#define hview_pixmap_set_func(hview, func) \
  (hview)->pixfunc = (HViewPixmapFunc)(func)
#define hview_pixmap_get(hview, apack, aver, data) \
  ((hview)->pixfunc)((apack), (aver), (data))

#define hview_is_ctree(hview) ((hview)->type == HVIEW_CTREE)
#define hview_is_clist(hview) ((hview)->type == HVIEW_CLIST)

#define hview_avail_get(hview) (hview)->avail
#define hview_avail_set(hview, list) (hview)->avail = (list)

#define hview_ctree_get(hview) GTK_CTREE((hview)->view)
#define hview_clist_get(hview) GTK_CLIST((hview)->view)
#define hview_widget_get(hview) (hview)->view

#define hview_freeze(hview) gtk_clist_freeze(hview_clist_get(hview))
#define hview_thaw(hview) gtk_clist_thaw(hview_clist_get(hview))

#ifdef __cplusplus
}
#endif

#endif /* _HVIEW_H */

