/* HickUP 1.2 -- Graphical interface to hick.
** Copyright (C) 1998 - 2001, 2008 Daniel Bergstrom <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtkmenu.h>
#include <gtk/gtkhbox.h>
#include <gtk/gtkframe.h>
#include <gtk/gtkdialog.h>
#include <gtk/gtkmenuitem.h>

#include "gui.h"
#include "menupop.h"

typedef struct {
  GUI_Dialog dialog;

  Project *project;
  PArr prjs;
  GtkWidget *clist;
  GtkWidget *ok_button;

  GUI_Dialog_Msg noprj_dialog;
} EditCopyPrj;

static void cb_edit_show_copy_prj(GtkWidget *w, Project *project);
static void cb_edit_copy_prj_ok(GtkWidget *w, EditCopyPrj *ecprj);

void
cb_select_clist(GtkWidget *clist, gint row, gint column, GdkEventButton *event,
                HView *hview)
{
  hview_select_set(hview,
                   (NodeTree *)gtk_clist_get_row_data(GTK_CLIST(clist), row));

}


void
cb_select_ctree(GtkWidget *ctree, GtkCTreeNode *row, gint column, HView *hview)
{
  hview_select_set(hview, 
                   (NodeTree *)gtk_ctree_node_get_row_data(GTK_CTREE(ctree), 
                                                           row));
}


gint
cb_edit_key_item(GtkWidget *w, GdkEventKey *event, HView *hview)
{
  AvailPacket *apack;
  AvailVersion *aver;

  if(!event || event->type != GDK_KEY_PRESS || 
     !hview_selected_get_avail(hview, &apack, &aver) || (!apack && !aver)) {
    return FALSE;
  }

  if(event->keyval == ' ') {
    avail_set(apack, aver, -1, TRUE, hview->avail);  
    return TRUE;
  }

  return FALSE;
}


gint
cb_edit_click_item(GtkWidget *w, GdkEventButton *event, HView *hview)
{
  gint row, column;
  AvailPacket *apack;
  AvailVersion *aver;

  if(!event || (event->type != GDK_BUTTON_PRESS)) {
    return FALSE;
  }

  hview_select_get_avail(hview, event, &apack, &aver, &row, &column);

  switch(event->button) {
    case 1: { /* step install type by one */
      if(column || (!apack && !aver)) {
        return FALSE;
      }

      avail_set(apack, aver, -1, TRUE, hview->avail);
      return TRUE;
    }
    case 3: { /* show popup menu */
      hview_select_row(hview, row);

      if(!apack) {
        hview_menu_set_show(hview, MENU_ALL, FALSE);
        hview_menu_set_show(hview, MENU_COPY_PRJ, TRUE);
      } else {
        hview_menu_set_show(hview, MENU_ALL, TRUE);

        if(aver) {
          hview_menu_set_gpixmap(hview, MENU_ENABLE, &_gmain.pixmaps.selected);
          hview_menu_set_label(hview, MENU_ENABLE, "Enable Version");

          hview_menu_set_show(hview, MENU_ENABLE_PACKET, TRUE);
          hview_menu_set_show(hview, MENU_DISABLE_PACKET, TRUE);
          hview_menu_set_show(hview, MENU_FORCE, TRUE);

          hview_menu_set_sensitive(hview, MENU_ENABLE,
                                   avail_can_enable(apack, aver, 
                                                    INSTALL_VERSION, TRUE,
                                                    hview->avail));
          hview_menu_set_sensitive(hview, MENU_FORCE,
                                   avail_can_enable(apack, aver, INSTALL_FORCE,
                                                    TRUE, hview->avail));
        } else {
          hview_menu_set_gpixmap(hview, MENU_ENABLE,&_gmain.pixmaps.dselected);
          hview_menu_set_label(hview, MENU_ENABLE, "Enable Packet");
          hview_menu_set_show(hview, MENU_ENABLE_PACKET, FALSE);
          hview_menu_set_show(hview, MENU_DISABLE_PACKET, FALSE);
          hview_menu_set_show(hview, MENU_FORCE, FALSE);

          hview_menu_set_sensitive(hview, MENU_ENABLE, 
                                   avail_can_enable(apack, NULL,
                                                    INSTALL_VERSION, TRUE,
                                                    hview->avail));
        }

        hview_menu_set_sensitive(hview, MENU_DEFAULT,  apack->can_def);
        hview_menu_set_sensitive(hview, MENU_DISABLE, TRUE);

        if(apack->packet->info.forced || (aver && aver->version->info.forced)){
          hview_menu_set_sensitive(hview, MENU_ALL,  FALSE);
        }

        hview_menu_set_sensitive(hview, MENU_VIEW, TRUE);
      }
    }

    if(hview->menu.menu) {
      gtk_menu_popup(GTK_MENU(hview->menu.menu), NULL, NULL, NULL, NULL, 
                     event->button, 0);
    }
      
    return TRUE;
  }

  return FALSE;
}

void
cb_edit_view(GtkWidget *w, HView *hview)
{
  if(hview->select.packet) {
    gui_find_expand(hview->select.packet->packet->name, 
                    (!hview->select.version ? NULL :
                     hview->select.version->version->name),
                    hview->select.packet->packet->type,
                    hview->project, TRUE);
  }
}

void
cb_edit_copy_prj(GtkWidget *w, HView *hview)
{
  if(_gmain.projects.projects.siz <= 2) {
    dialog_err_return_new(0, "Copy Project Specification",
                          "No availible project to copy from.");
  } else {
    cb_edit_show_copy_prj(0, _gmain.projects.current);
  }
}

static void
cb_edit_show_copy_prj(GtkWidget *w, Project *project)
{
  size_t n;
  gchar *titles[1];
  Project *prj;
  EditCopyPrj *ecprj;
  GtkWidget *hbox, *frame, *box, *label;

  if(!project->edit->copy_from) {
    project->edit->copy_from = ecprj = MallocMem(EditCopyPrj);
    ecprj->project = project;
    dialog_msg_init(&ecprj->noprj_dialog);

    dialog_new(&ecprj->dialog, "HickUP Copy Project Specification",
               "OK", BUTTON_ISDEF, &ecprj->ok_button,cb_edit_copy_prj_ok,ecprj,
               "Cancel", BUTTON_CANDEF, NULL, NULL, NULL,
               NULL);

    gtk_window_set_position(GTK_WINDOW(ecprj->dialog.window),
                            GTK_WIN_POS_CENTER);

    hbox = gtk_hbox_new(FALSE, 5);
    gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(ecprj->dialog.window)->vbox), hbox,
                       TRUE, TRUE, 0);

    frame = gtk_frame_new(NULL);
    gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, TRUE, 0);

    box = gtk_hbox_new(FALSE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(box), 5);
    gtk_container_add(GTK_CONTAINER(frame), box);

    label = gtk_label_new("Choose project to copy specification from. "
                          "WARNING: this will replace all current "
                          "settings.");

    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
    gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_FILL);
    gtk_box_pack_start(GTK_BOX(box), label, FALSE, TRUE, 0);

    titles[0] = "Project";

    ecprj->clist =  gtk_clist_new_with_titles(1, titles);
    gtk_widget_set_usize(ecprj->clist, -1, 100);
    gtk_box_pack_start(GTK_BOX(hbox), ecprj->clist, TRUE, TRUE, 0);
    gtk_clist_column_titles_passive(GTK_CLIST(ecprj->clist));
    gtk_clist_set_column_min_width(GTK_CLIST(ecprj->clist), 0, 120);
    gtk_clist_set_column_auto_resize(GTK_CLIST(ecprj->clist), 0, TRUE);
    gtk_clist_set_selection_mode(GTK_CLIST(ecprj->clist),
                                 GTK_SELECTION_MULTIPLE);

    parr_init(&ecprj->prjs);

    for(n = 1; n < _gmain.projects.projects.siz; n++) {
      prj = parr_get(&_gmain.projects.projects, Project, n);

      if(prj != project) {
        titles[0] = prj->name;
        gtk_clist_append(GTK_CLIST(ecprj->clist), titles);
        parr_add(&ecprj->prjs, prj); 
      }
    }
  } else {
    ecprj = (EditCopyPrj *)project->edit->copy_from;

    if(ecprj->noprj_dialog.dialog.window) {
      gtk_widget_hide(ecprj->noprj_dialog.dialog.window);
    }
  }
 
  gtk_widget_grab_focus(ecprj->ok_button);
  gtk_widget_show_all(ecprj->dialog.window);
}

static void
cb_edit_copy_prj_ok(GtkWidget *w, EditCopyPrj *ecprj)
{
  gint row;
  Project *prj;

  gtk_widget_hide(ecprj->dialog.window);

  if(GTK_CLIST(ecprj->clist)->selection) {
    row = GPOINTER_TO_INT(GTK_CLIST(ecprj->clist)->selection->data);
    prj = parr_get(&ecprj->prjs, Project, row);

    /* reinitialize 'edit' tab */

    select_clear(ecprj->project->edit);
    reqchk_close(&ecprj->project->edit->reqchk);
    hview_freeze(&ecprj->project->edit->hview);
    hview_display_destroy(&ecprj->project->edit->hview);
    avail_free(&ecprj->project->edit->avail);

    ecprj->project->edit->has_init = 0;

    project_init_edit(ecprj->project, &_gmain.projects.packets,
                      &_gmain.projects.types, prj->specfile);

    hview_thaw(&ecprj->project->edit->hview);

    gtk_clist_unselect_row(GTK_CLIST(ecprj->clist), row, 0);
  } else {
    if(!ecprj->noprj_dialog.dialog.window) {
      dialog_msg_new(&ecprj->noprj_dialog,
                     "Copy Project Specification",
                     "No project selected to be copied.",
                     "OK", BUTTON_ISDEF, NULL, cb_edit_show_copy_prj,
                     ecprj->project, 
                     "Cancel", BUTTON_CANDEF, NULL, NULL, NULL,
                     NULL);
    }
    
    gtk_widget_show_all(ecprj->noprj_dialog.dialog.window);
  }
}

void
cb_edit_enable(GtkWidget *w, HView *hview)
{
  if(hview->select.packet) {
    avail_set(hview->select.packet, hview->select.version, 
              (hview->select.version ? INSTALL_VERSION : INSTALL_DEFAULT), 
              TRUE, hview->avail);
  }
}


void
cb_edit_enable_packet(GtkWidget *w, HView *hview)
{
  if(hview->select.packet) {
    avail_set(hview->select.packet, NULL, INSTALL_DEFAULT, TRUE, hview->avail);
  }
}


void
cb_edit_disable_packet(GtkWidget *w, HView *hview)
{
  if(hview->select.packet) {
    avail_set(hview->select.packet, NULL, INSTALL_NOT, TRUE, hview->avail);
  }
}


void
cb_edit_enable_default(GtkWidget *w, HView *hview)
{
  size_t n;
  AvailVersion *aver;

  if(!hview->select.packet || !hview->select.packet->can_def) {
    return;
  }

  for(n = 0; n < hview->select.packet->vers.siz; n++) {
    aver = parr_get(&hview->select.packet->vers, AvailVersion, n);

    avail_set(hview->select.packet, aver, 
              (aver->version->isdef ? INSTALL_VERSION : INSTALL_NOT), 
              TRUE, hview->avail);
  }
}


void
cb_edit_disable(GtkWidget *w, HView *hview)
{
  if(hview->select.packet) {
    avail_set(hview->select.packet, hview->select.version, INSTALL_NOT, TRUE, 
              hview->avail);
  }
}


void
cb_edit_force(GtkWidget *w, HView *hview)
{
  if(hview->select.packet && hview->select.version) {
    avail_set(hview->select.packet, hview->select.version, INSTALL_FORCE, 
              TRUE, hview->avail);
  }
}


gint
cb_pack_key_item(GtkWidget *w, GdkEventKey *event, HView *hview)
{
  AvailPacket *apack;
  AvailVersion *aver;

  if(!event || event->type != GDK_KEY_PRESS || 
     !hview_selected_get_avail(hview, &apack, &aver) || (!apack && !aver)) {
    return FALSE;
  }

  if(event->keyval == ' ') {
    avail_set(apack, aver, -1, FALSE, hview->avail);  
    return TRUE;
  }

  return FALSE;
}


gint
cb_pack_click_item(GtkWidget *w, GdkEventButton *event, HView *hview)
{
  gint row, column;
  AvailVersion *aver;
  AvailPacket *apack, *apfpack;

  if(!event || (event->type != GDK_BUTTON_PRESS) || 
     !hview_select_get_avail(hview, event, &apack, &aver, &row, &column)) {
    return FALSE;
  }

  switch(event->button) {
    case 1: {                   /* step install type by one */
      if(column || (!apack && !aver)) {
        return FALSE;
      }

      avail_set(apack, aver, -1, FALSE, hview->avail);
      return TRUE;
    }
    case 3: {                   /* show popup menu */
      hview_select_row(hview, row);

      if(apack) {
        hview_menu_set_sensitive(hview, MENU_DEFAULT,  TRUE);
        hview_menu_set_sensitive(hview, MENU_DISABLE_PACKET,  TRUE);

        if(aver) {
          hview_menu_set_show(hview, MENU_ENABLE, TRUE);   
          hview_menu_set_show(hview, MENU_DISABLE, TRUE);
        } else {
          hview_menu_set_show(hview, MENU_ENABLE, FALSE);   
          hview_menu_set_show(hview, MENU_DISABLE, FALSE);
        }

        if(apack->packet->info.forced || (aver && aver->version->info.forced)){
          hview_menu_set_sensitive(hview, MENU_ALL,  FALSE);
        } else {
          if((apfpack= avail_find_packet(apack->packet->name, hview->avail))) {
            if(apfpack->info.install == INSTALL_FORCE) {
              hview_menu_set_sensitive(hview, MENU_ALL,  FALSE);
            }
          }
        }

        hview_menu_set_sensitive(hview, MENU_VIEW, TRUE);

        if(hview->menu.menu) {
          gtk_menu_popup(GTK_MENU(hview->menu.menu), NULL, NULL, NULL, NULL, 
                         event->button, 0);
        }
      }

      return TRUE;
    }
  }

  return FALSE;
}


void 
cb_pack_view(GtkWidget *w, HView *hview)
{
  if(hview->select.packet) {
    gui_find_expand(hview->select.packet->packet->name, 
                    (!hview->select.version ? NULL :
                     hview->select.version->version->name),
                    hview->select.packet->packet->type,
                    hview->project, FALSE);
    
    
  }
}


void
cb_pack_select(GtkWidget *w, HView *hview)
{
  if(hview->select.packet && hview->select.version) {
    avail_set(hview->select.packet, hview->select.version, INSTALL_VERSION, 
              FALSE, hview->avail);
  }
}


void
cb_pack_select_default(GtkWidget *w, HView *hview)
{
  if(hview->select.packet) {
    avail_set(hview->select.packet, NULL, INSTALL_DEFAULT, FALSE, 
              hview->avail);
  }
}


void
cb_pack_unselect(GtkWidget *w, HView *hview)
{
  if(hview->select.packet && hview->select.version) {
    avail_set(hview->select.packet, hview->select.version, INSTALL_NOT,
              FALSE, hview->avail);
  }
}


void
cb_pack_unselect_packet(GtkWidget *w, HView *hview)
{
  if(hview->select.packet) {
    avail_set(hview->select.packet, NULL, INSTALL_NOT, FALSE, hview->avail);
  }
}
