/* HickUP 1.2 -- Graphical interface to hick.
** Copyright (C) 1998 - 2001, 2008 Daniel Bergstrom <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <string.h>

#include "hview.h"


Node *
node_move(Node *node, NodeList *from, NodeList *to)
{
  if(hsh_node_remove(node, from)) {
    return hsh_node_append(node, to);
  }

  return NULL;
}


Node *
node_find(const char *name, NodeList *list)
{
  Node *node;

  for(node = list->head; node; node = node->next) {
    if(!strcasecmp(name, node->name)) {
      return node;
    }
  }

  return NULL;
}


Node *
node_find_recursive(const char *name, unsigned int type, NodeList *list)
{
  Node *node, *rnode;

  for(node = list->head; node; node = node->next) {
    if((node->flag != type) && 
       ((node->flag != NT_VERSION) && (node->flag != NT_EMPTY))) {
      
      if((rnode = node_find_recursive(name, type, &((NTList *)node)->nlist))) {
        return rnode;
      }
    } else {
      if(!strcasecmp(name, node->name)) {
        return node;
      }
    }
  }

  return NULL;
}


Node *
node_insert(Node *node, NodeList *to)
{
  Node *n;

  if(node) {
    for(n = to->head; n; n = n->next) {
      if(strcasecmp(node->name, n->name) <= 0) {
        node->prev = n->prev;
        node->next = n;
        n->prev = node;

        if(node->prev) {
          node->prev->next = node;
        } else {
          to->head = node;
        }

        return node;
      }
    }

    hsh_node_append(node, to);
  }

  return node;
}
