/* HickUP 1.2 -- Graphical interface to hick.
** Copyright (C) 1998 - 2001, 2008 Daniel Bergstrom <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _PACKET_H
#define _PACKET_H

#include "node.h"
#include "array.h"

#define GET_VERSION(parr, n)  ((Version *)(arr)[n])
#define GET_PACKETREF(parr, n)  ((PacketRef *)(arr)[n])

#define NT_EMPTY    0
#define NT_TYPE     1
#define NT_GROUP    2
#define NT_PACKET   3
#define NT_VERSION  4

typedef struct _packet Packet;

/* RCInfo ****************************************************************** */

typedef struct {
  Packet *packet;

  PArr vers; /* Version */
} RCInfo;

/* Require ***************************************************************** */

typedef struct _require Require;

struct _require {
  RCInfo info;

  Require *next;
  Require *ror;
};

/* Conflict **************************************************************** */

typedef RCInfo Conflict;

/* SubGroup **************************************************************** */

typedef struct {
  Node node;

  unsigned int here : 1; /* have a packet here? */

  NodeList down; /* list of groups below this */
} SubGroup;

/* PVInfo ****************************************************************** */

typedef struct {
  char *about;

  time_t created;
  time_t modified;
  time_t expire;

  PArr conflicts; /* Conflict */

  Require *require; /* tree of required packets */

  unsigned int forced : 1;
} PVInfo;

/* Packet ****************************************************************** */

struct _packet {
  Node node;

  char name[PNAME_LEN + 1];
  char *type;

  PArr groups; /* char * */
  NodeList subgroups; /* list of subgroups */

  PArr vers; /* Version */

  PVInfo info;

  unsigned int hidden   : 1;
  unsigned int required : 1;
  unsigned int multiple : 1;
};

/* Version ***************************************************************** */

typedef struct {
  Packet *packet;

  char name[PVER_LEN + 1];

  PVInfo info;

  unsigned int override : 1; /* override the packets about message? */  
  unsigned int isdef    : 1; /* is this version the default? */
  unsigned int real     : 1; /* Is real version? (used when loading) */
} Version;

#ifdef __cplusplus
extern "C" {
#endif

extern Packet *packet_new(HickPacket *hpac, const char *name, 
                          const char *type, PArr *groups, NodeList *emptyList, 
                          NodeList *packList);
extern Packet *packet_find(const char *name, const char *type, NodeList *list);
extern void packets_clear(NodeList *list);

extern Version *version_new(HickVer *hver, const char *name, Packet *packet);

#define version_find(parr, name) \
  parr_find_value((parr), Version, (name), _versioncmp)

extern int _versioncmp(const void *d1, const void *d2);

#ifdef __cplusplus
}
#endif

#endif /* _PACKET_H */
