/* HickUP 1.2 -- Graphical interface to hick.
** Copyright (C) 1998 - 2001, 2008 Daniel Bergstrom <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _PROJECT_H
#define _PROJECT_H

#ifndef _HVIEW_H
typedef struct _hup_project Project;
#endif /* _HVIEW_H */

typedef struct _hup_projects Projects;

#include "tabs.h"
#include "node.h"
#include "dialog.h"

#ifndef PVER_LEN
#define PVER_LEN   16           /* max verison length of a packet */
#endif

#ifndef PNAME_LEN
#define PNAME_LEN  32           /* max name length of a packet */
#endif

#define HICK_SPECFILE  "profile.spec"
#define HICK_INFOFILE  "profile.desc"

/* Removed ***************************************************************** */

typedef struct {
  char packet[PNAME_LEN + 1]; /* removed packet / version, by project spec */
  char version[PVER_LEN + 1];
  char *type;
} Removed;

/* Project ***************************************************************** */

struct _hup_project {
  char *name;                   /* Name of project (name on tab) */
  char *prjdir;                 /* project directory */
  char *specfile;               /* where the specifications are stored */
  char *infofile;               /* project description file */
  char *homedir;                /* home directory for hick */

  unsigned int can_edit : 1;    /* can the user edit the project */

  GtkWidget *notebook;

  TabPackets packets;
  TabFilter filter;
  TabPackets *edit;

  PArr removed;                 /* (Removed *) */

  GUI_Dialog report;            /* Packet report */
  GUI_Dialog update;            /* Update report */
};

/* Projects **************************************************************** */

struct _hup_projects {
  PArr types;                   /* where the type names are allocated */
  PArr groups;                  /* where the group names are allocated */
  NodeList packets;             /* list of all packets */
  NodeList failure;             /* packets which failed to load */

  PArr names;                   /* names on the projects. */
  PArr projects;                /* array of pointers to projects */

  char *hicktype;               /* current hick type, can be NULL */

  Project *current;             /* pointer to the current project */

  GtkWidget *notebook;
};

#ifdef __cplusplus
extern "C" {
#endif

extern void projects_init(Projects *prjs, const char *prjsstr, char *user, 
                          char *homedir, char *userdesc);

extern int projects_get_types(Projects *prjs, const char *path);

extern void project_init_packets(Project *prj, NodeList *packets, PArr *types);
extern void project_init_edit(Project *prj, NodeList *packets, PArr *types,
                              const char *specfile);

extern void project_removed(Project *prj, char *name, char *version,
                            char *type);

extern void projects_clear_all(Projects *prjs);
extern void cb_reload_projects(GtkWidget *w, gpointer data);

/* load.c */

extern int projects_load_packets(Projects *prjs, const char *path);

#ifdef __cplusplus
}
#endif

#endif /* _PROJECT_H */
