/* HickUP 1.2 -- Graphical interface to hick.
** Copyright (C) 1998 - 2001, 2008 Daniel Bergstrom <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _REFS_H
#define _REFS_H

#ifndef _HVIEW_H
typedef struct _hup_nodetree NodeTree;
#endif /* _HVIEW_H */

#include <gtk/gtkctree.h>

#include "hview.h"

/* NodeTree **************************************************************** */

struct _hup_nodetree {
  Node node;

  NodeTree *parent;
  GtkCTreeNode *ctnode;
};

/* NTList ****************************************************************** */

typedef struct {
  NodeTree ntree;

  NodeList nlist;
} NTList;

/* RefType and RegGroup **************************************************** */

typedef NTList RefType;
typedef NTList RefGroup;

/* RefPacket *************************************************************** */

typedef struct {
  NTList ntlist;

  HView *hview;

  AvailPacket *packet;
} RefPacket;

/* RefVersion ************************************************************** */

typedef struct {
  NodeTree ntree;

  AvailVersion *version;
} RefVersion;

/* RefData ***************************************************************** */

typedef struct {
  NTList ntlist;

  HView *hview;

  gint row;

  AvailPacket *packet;
  AvailVersion *version;
} RefData;

#ifdef __cplusplus
extern "C" {
#endif

#define nodetree_init(nt)   \
  node_init(&(nt)->node), (nt)->parent = NULL, (nt)->ctnode = NULL

#define ntlist_init(ntl) \
  nodetree_init(&(ntl)->ntree), hsh_nodelist_init(&(ntl)->nlist)

#define ntlist_find(n, l)  (NTList *)node_find((n), (l))


extern RefType *refs_lookup_type(const char *name, NodeList *list);
extern RefGroup *refs_lookup_group(char *name, NTList *parent);

extern RefPacket *refs_packet_new(char *name, AvailPacket *packet, 
                                  NTList *parent);

extern RefVersion *refs_version_new(char *name, AvailVersion *ver, 
                                    RefPacket *rp);

extern RefData *refs_data_new(char *name, AvailPacket *packet, 
                              AvailVersion *version, NodeList *list);

extern void refs_free(NodeList *list);

extern void refs_display_tree(NodeList *list, HView *hview, 
                              GtkCTreeNode *parent, int expandall);

#ifdef __cplusplus
}
#endif

#endif /* _REFS_H */
