/* HickUP 1.2 -- Graphical interface to hick.
** Copyright (C) 1998 - 2001, 2008 Daniel Bergstrom <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _REPORT_H
#define _REPORT_H

#include <sys/types.h>

#include <gtk/gtkwidget.h>

typedef enum {
  RTYPE_NONE, RTYPE_CLIST
} ReportType;

typedef struct _report Report;
typedef struct _reporttab ReportTab;

typedef void (*ReportCreate)(Report *r, ReportTab *rt, GtkWidget *vbox);
typedef void (*ReportClear)(Report *r, ReportTab *rt);
typedef void (*ReportSignal)(GtkWidget *w, gint row, gint column,
                             GdkEventButton *event, gpointer data);

/* ReportColumn ************************************************************ */

typedef struct {
  gint *length;   /* array of min length of columns */
  gchar **titles; /* array of column titles */
} ReportColumn;

/* ReportTab *************************************************************** */

struct _reporttab {
  GtkWidget *name;     /* tab name */
  GtkWidget *pixmap;   /* tab pixmap */

  ReportClear clear;   /* clear function */
  ReportCreate create; /* create function */

  ReportType type;

  int got_items;

  void *arg; /* for user data when this is being passed */
};

/* ReportTabCList ********************************************************** */

typedef struct {
  ReportTab rtab;

  GtkWidget *clist; /* clist with info */

  ReportColumn columns; /* column information */
  size_t colSize; /* size of columns */

  GtkWidget *help; /* short help text; */

  ReportSignal sig_func; /* function to connect to double click */
} ReportTabCList;

/* Report ****************************************************************** */

struct _report {
  ReportTab **tabs; /* array of tabs */
  size_t tabSize; /* size of tabs */

  GtkWidget *notebook;

  GdkPixmap *pix;
  GdkBitmap *mask;
};

#ifdef __cplusplus
extern "C" {
#endif

extern Report *report_new(size_t tabs, GdkPixmap *pix, GdkBitmap *mask);

extern void report_tab_set(Report *r, size_t tabnr, const char *name,
                           ReportCreate create, ReportClear clear, void *arg);

extern void report_tab_set_clist(Report *r, size_t tabnr, const char *name,
                                 const char *help, ReportSignal func, 
                                 void *arg, size_t columns);

extern void report_tab_clist_col_set(Report *r, size_t tabnr, size_t column,
                                     const gchar *title, gint length);

extern GtkWidget *report_create(Report *r);
extern void report_clear(Report *r);

extern void report_show_pixmaps(Report *r);
extern int report_display(Report *r, unsigned int mask);

extern gint report_add_clist_item(Report *r, size_t tabnr, ...);

#ifdef __cplusplus
}
#endif

#endif /* _REPORT_H */
