/* HickUP 1.2 -- Graphical interface to hick.
** Copyright (C) 1998 - 2001, 2008 Daniel Bergstrom <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gtk/gtkhbox.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkdialog.h>

#include "gui.h"
#include "wwrap.h"
#include "reqchk.h"

static void reqchk_new(ReqChk *rchk);

static RefData *reqchk_require(ReqChk *rchk, AvailPacket *apack, 
                               AvailVersion *aver, NodeList *avail,
                               Require *head, RefData *rdat);

static RefData *reqchk_refdata_new(AvailPacket *apack, AvailVersion *aver, 
                                   int isor, NodeList *list);

static void cb_packets_select(GtkCList *clist, gint row, gint column, 
                              GdkEvent *event, ReqChk *rchk);


void
reqchk_clear(ReqChk *rchk)
{
  if(rchk->dialog.window) {
    reqchk_set(&rchk->packets.display, FALSE, FALSE);
    reqchk_set(&rchk->require.display, FALSE, TRUE);

    hview_display_destroy(&rchk->packets);

    hview_display_clear(&rchk->require);
    hsh_nodelist_init(&rchk->require.display);
  }
}


int
reqchk_show(ReqChk *rchk)
{
  if(rchk->dialog.window && rchk->packets.display.head) {
    gtk_clist_select_row(hview_clist_get(&rchk->packets), 0, -1);
    gtk_widget_show(rchk->dialog.window);
    return 1;
  }

  return 0;
}


void
reqchk_close(ReqChk *rchk)
{
  if(rchk->dialog.window) {
    reqchk_clear(rchk);
    gtk_widget_hide(rchk->dialog.window);
  }
}


void
reqchk_check(ReqChk *rchk, int isedit)
{
  size_t n;
  Node *node;
  RefData *rpack;
  AvailVersion *aver;

  for(node = rchk->packets.avail->head; node; node = node->next) {
    if(!((AvailPacket *)node)->info.install) {
      continue;
    }

    for(n = 0; n < ((AvailPacket *)node)->vers.siz; n++) {
      aver = parr_get(&((AvailPacket *)node)->vers, AvailVersion, n);

      if(aver->info.install ||
         (isedit && (((AvailPacket *)node)->info.install == INSTALL_DEFAULT))){

        rpack = reqchk_require(rchk, (AvailPacket *)node, aver, 
                               rchk->packets.avail,
                               ((AvailPacket *)node)->packet->info.require,
                               NULL);

        reqchk_require(rchk, (AvailPacket *)node, aver, rchk->packets.avail,
                       aver->version->info.require, rpack);
      }
    }
  }
}


int 
reqchk_check_edit(Require *req, NodeList *avail)
{
  size_t n;
  Require *nreq;
  AvailPacket *apack;
  AvailVersion *aver;

  if((apack = avail_find_packet(req->info.packet->name, avail))) {
    if(!req->info.vers.siz && apack->info.install) {
      return 1; /* got a version of that packet. */
    }

    for(n = 0; n < req->info.vers.siz; n++) { /* check for a version */
      aver = avail_find_version(parr_get(&req->info.vers, Version, n)->name, 
                                apack);

      if((apack->info.install == INSTALL_DEFAULT) || 
         (aver && aver->info.install)) {
        return 1; /* got a valid version */
      }
    }
  }

  if(req->ror) { /* check or statment for valid require */
    for(nreq = req->ror; nreq; nreq = nreq->next) {
      if(reqchk_check_edit(nreq, avail)) {
        return 1;
      }
    }
  }

  return 0;
}


int 
reqchk_check_packets(Require *req, NodeList *avail)
{
  size_t n;
  Require *nreq;
  AvailPacket *apack;
  AvailVersion *aver;

  if((apack = avail_find_packet(req->info.packet->name, avail))) {
    if(!req->info.vers.siz && apack->info.install) {
      return 1; /* got a version of that packet. */
    }

    for(n = 0; n < req->info.vers.siz; n++) { /* check for a version */
      aver = avail_find_version(parr_get(&req->info.vers, Version, n)->name, 
                                apack);

      if(aver && aver->info.install) {
        return 1; /* got a valid version */
      }
    }
  }

  if(req->ror) { /* check or statment for valid require */
    for(nreq = req->ror; nreq; nreq = nreq->next) {
      if(reqchk_check_edit(nreq, avail)) {
        return 1;
      }
    }
  }

  return 0;
}


GPixmap *
reqchk_pixmap(AvailPacket *apack, AvailVersion *aver, void *data)
{
  unsigned int install;

  if(apack) {
    install = (aver && (apack->info.install != INSTALL_DEFAULT) ?
               aver->info.install : apack->info.install);

    switch(install) {
      case INSTALL_NOT: {
        return &_gmain.pixmaps.select;
      }
      case INSTALL_DEFAULT: {
        return &_gmain.pixmaps.dselected;
      }
      case INSTALL_FORCE: {
        return &_gmain.pixmaps.forced;
      }
      case INSTALL_VERSION: {
        return &_gmain.pixmaps.selected;
      }
    }
  }

  return &_gmain.pixmaps.attention;
}


static void
reqchk_new(ReqChk *rchk)
{ 
  gint cwidth[4];
  gchar *titles[4];
  NodeList *list;
  GtkWidget *w, *vbox, *hbox;

  dialog_new(&rchk->dialog, rchk->title,
             "OK", BUTTON_ISDEF, NULL, rchk->buttons[0].func, 
             rchk->buttons[0].data,
             "Ignore", BUTTON_CANDEF, NULL, rchk->buttons[1].func, 
             rchk->buttons[1].data,
             "Cancel", BUTTON_CANDEF, NULL, rchk->buttons[2].func, 
             rchk->buttons[2].data,
             NULL);

  gtk_widget_set_usize(rchk->dialog.window, 550, 300);

  vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(rchk->dialog.window)->vbox), vbox, 
                     TRUE, TRUE, 0);

  ww_frame(NULL, NULL, rchk->help, NULL, vbox);

  hbox = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

  cwidth[0] = 16;
  cwidth[1] = 75;
  cwidth[2] = 75;
  cwidth[3] = -1;
  titles[0] = "";
  titles[1] = "Packet";
  titles[2] = "Version";
  titles[3] = "For Type";
  list = rchk->packets.avail;
  w = hview_new(&rchk->packets, HVIEW_CLIST, 4, titles, cwidth, 
                rchk->packets.project);
  rchk->packets.avail = list;
  gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);

  gtk_signal_connect(GTK_OBJECT(rchk->packets.view), "select_row", 
                     GTK_SIGNAL_FUNC(cb_packets_select), rchk);

  cwidth[1] = -1;
  titles[1] = "Requiers";
  list = rchk->require.avail;
  w = hview_new(&rchk->require, HVIEW_CTREE, 2, titles, cwidth,
                rchk->require.project);
  rchk->require.avail = list;
  gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 0);
  gtk_widget_show_all(vbox);

  hview_pixmap_set_func(&rchk->packets, reqchk_pixmap);
  hview_pixmap_set_func(&rchk->require, reqchk_pixmap);

  if(rchk->newfunc) {
    (rchk->newfunc)(rchk);
  }
}


void
reqchk_set(NodeList *list, int add, int recursive)
{
  size_t n;
  Node *node;

  for(node = list->head; node; node = node->next) {
    if(add) {
      parr_add(&((RefData *)node)->packet->refs, (NTList *)node);
    } else {
      if((n = parr_find(&((RefData *)node)->packet->refs, (NTList *)node,0))) {
        parr_remove(&((RefData *)node)->packet->refs, n - 1);
      }

      ((RefData *)node)->row = -1;
      ((RefData *)node)->ntlist.ntree.ctnode = NULL;
    }

    if(recursive) {
      reqchk_set(&((NTList *)node)->nlist, add, recursive);
    }
  }
}


static RefData *
reqchk_require(ReqChk *rchk, AvailPacket *apack, AvailVersion *aver, 
               NodeList *avail, Require *head, RefData *rdat)
{
  Require *req;
  GPixmap *pixi;
  gchar *text[4];

  for(text[0] = (char)0, req = head; req; req = req->next) {
    if((rchk->testfunc)(req, avail)) {
      continue;
    }

    if(!rchk->dialog.window) {
      reqchk_new(rchk);
    }

    if(!rdat) {
      rdat = reqchk_refdata_new(apack, aver, -1, &rchk->packets.display);
      rdat->hview = &rchk->packets;
      parr_add(&apack->refs, &rdat->ntlist);

      text[1] = apack->packet->name;
      text[2] = aver->version->name;
      text[3] = apack->packet->type;

      rdat->row = gtk_clist_append(hview_clist_get(&rchk->packets), text);
      gtk_clist_set_row_data(hview_clist_get(&rchk->packets), rdat->row, rdat);

      pixi = hview_pixmap_get(&rchk->packets, apack, aver, NULL);
      gtk_clist_set_pixmap(hview_clist_get(&rchk->packets), rdat->row, 0,
                           pixi->pix, pixi->mask);
    }

    reqchk_create_require(&rchk->require, req, avail, &rdat->ntlist.nlist, 
                          FALSE);
  }

  return rdat;
}


static RefData *
reqchk_refdata_new(AvailPacket *apack, AvailVersion *aver, int isor, 
                   NodeList *list)
{
  size_t n;
  char *str;

  if(isor == -1) {
    str = NULL;
  } else {
    n = (strlen(apack->packet->name) + 
         (aver ? strlen(aver->version->name) + 1 : 0) + (isor ? 4 : 1));
 
    str = MallocArr(char, n);

    if(isor) {
      if(aver) {
        hsh_create_str(str, n, "or ", apack->packet->name, " ", 
                       aver->version->name, NULL);
      } else {
        hsh_create_str(str, n, "or ", apack->packet->name, NULL);
      }
    } else {
      if(aver) {
        hsh_create_str(str, n, apack->packet->name, " ",aver->version->name,
                       NULL);
      } else {
        strcpy(str, apack->packet->name);
      }
    }
  }

  return refs_data_new(str, apack, aver, list);
}


void
reqchk_create_require(HView *hview, Require *req, NodeList *list, NodeList *to,
                      int isor)
{
  size_t n;
  Require *nreq;
  AvailPacket *apack;
  AvailVersion *aver;
  RefData *rhead = NULL, *rnew;

  if((apack = avail_find_packet(req->info.packet->name, list))) {
    if(!req->info.vers.siz) {
      rhead = rnew = reqchk_refdata_new(apack, NULL, isor, to);
      rnew->hview = hview;
    } else {
      for(n = 0; n < req->info.vers.siz; n++) {
        aver = avail_find_version(parr_get(&req->info.vers, Version, n)->name, 
                                  apack);
        if(aver) {
          if(!isor && rhead) {
            rnew = reqchk_refdata_new(apack, aver, TRUE, &rhead->ntlist.nlist);
            rnew->hview = hview;
          } else {
            rhead = rnew = reqchk_refdata_new(apack, aver, isor, to);
            rnew->hview = hview;
          }
        }
      }
    }
  }

  if(rhead && !isor && req->ror) {
    for(nreq = req->ror; nreq; nreq = nreq->next) {
      reqchk_create_require(hview, nreq, list, &rhead->ntlist.nlist, TRUE);
    }
  }
}


static void
cb_packets_select(GtkCList *clist, gint row, gint column, GdkEvent *event,
                  ReqChk *rchk)
{
  RefData *rdat;

  if(!(rdat = (RefData *)gtk_clist_get_row_data(clist, row))) {
    return;
  }

  if((rdat->ntlist.nlist.head == rchk->require.display.head) &&
     (rdat->ntlist.nlist.tail == rchk->require.display.tail)) {
    return; /* already showing it */
  }

  hview_freeze(&rchk->require);
  gtk_clist_clear(hview_clist_get(&rchk->require));

  reqchk_set(&rchk->require.display, FALSE, TRUE);
  hsh_nodelist_init(&rchk->require.display);

  rchk->require.display.head = rdat->ntlist.nlist.head;
  rchk->require.display.tail = rdat->ntlist.nlist.tail;

  refs_display_tree(&rchk->require.display, &rchk->require, NULL, TRUE);
  reqchk_set(&rchk->require.display, TRUE, TRUE);

  hview_thaw(&rchk->require);
}
