/* HickUP 1.2 -- Graphical interface to hick.
** Copyright (C) 1998 - 2001, 2008 Daniel Bergstrom <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _REQCHK_H
#define _REQCHK_H

#include "dialog.h"
#include "avail.h"

typedef struct _hup_reqchk ReqChk;

typedef void (*ReqChkNew)(ReqChk *rchk);
typedef int (*ReqChkTest)(Require *req, NodeList *avail);

/* ReqChkButton ************************************************************ */

typedef struct {
  GtkSignalFunc func;
  gpointer data;
} ReqChkButton;

/* ReqChk ****************************************************************** */

struct _hup_reqchk {
  GUI_Dialog dialog;

  HView packets;
  HView require;

  const char *help;
  const char *title;

  ReqChkButton buttons[3];

  ReqChkNew newfunc; /* additional things to run at creation */
  ReqChkTest testfunc;
};

#ifdef __cplusplus
extern "C" {
#endif

#define reqchk_init(rchk, alist, prj, nfunc, tfunc)  \
   dialog_init(&(rchk)->dialog); (rchk)->newfunc = (ReqChkNew)nfunc; \
   (rchk)->testfunc = (ReqChkTest)tfunc; \
   (rchk)->packets.avail = (rchk)->require.avail = (alist); \
   (rchk)->packets.project = (rchk)->require.project = (prj)
    

#define reqchk_set_title(rchk, tstr)  (rchk)->title = (tstr)
#define reqchk_set_help(rchk, hstr)   (rchk)->help = (hstr)

#define reqchk_set_button_ok(rchk, signal, d) \
   (rchk)->buttons[0].func = (GtkSignalFunc)(signal); \
   (rchk)->buttons[0].data = (d)

#define reqchk_set_button_ignore(rchk, signal, d) \
   (rchk)->buttons[1].func = (GtkSignalFunc)(signal); \
   (rchk)->buttons[1].data = (d)

#define reqchk_set_button_cancel(rchk, signal, d) \
   (rchk)->buttons[2].func = (GtkSignalFunc)(signal); \
   (rchk)->buttons[2].data = (d)

extern void reqchk_clear(ReqChk *rchk);
extern int reqchk_show(ReqChk *rchk);
extern void reqchk_close(ReqChk *rchk);

extern void reqchk_check(ReqChk *rchk, int isedit);

extern GPixmap *reqchk_pixmap(AvailPacket *apack, AvailVersion *aver, 
                              void *data);

extern int reqchk_check_edit(Require *req, NodeList *avail);
extern int reqchk_check_packets(Require *req, NodeList *avail);

extern void reqchk_set(NodeList *list, int add, int recursive);
extern void reqchk_create_require(HView *hview, Require *req, NodeList *list, 
                                  NodeList *to, int isor);


#ifdef __cplusplus
}
#endif

#endif /* _REQCHK_H */
