/* HickUP 1.2 -- Graphical interface to hick.
** Copyright (C) 1998 - 2001, 2008 Daniel Bergstrom <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _SPECVARS_H
#define _SPECVARS_H

#include "node.h"
#include "array.h"

#define SPEC_FORCE  0
#define SPEC_ENABLE 1

/* SpecVar ***************************************************************** */

typedef struct {
  Node node;

  char name[PNAME_LEN + 1];

  PArr versions; /* (char *) */
} SpecVar;

/* SpecType **************************************************************** */

typedef struct {
  Node node;

  NodeList enable;
  NodeList force;
} SpecType;

/* SpecVars **************************************************************** */

typedef struct {
  NodeList types;
} SpecVars;

#ifdef __cplusplus
extern "C" {
#endif

#define spec_vars_init(svar)   hsh_nodelist_init(&(svar)->types)

#define spec_find_type(type, svars) \
  (SpecType *)node_find((type), &(svars)->types)

extern SpecVar *spec_find(const char *type, const char *name, int enable, 
                          SpecVars *svars);

extern int spec_load(const char *filename, SpecVars *result, int print_errors);
extern int spec_save(const char *filename, SpecVars *result);

extern int spec_add(const char *type, const char *name, PArr *versions, 
                    int enable, SpecVars *svars);

extern SpecVars *spec_free(SpecVars *svars);

extern SpecVar *stype_find(const char *name, int enable, SpecType *stype);

extern int specvar_is_version(SpecVar *svar, const char *version);

#ifdef __cplusplus
}
#endif

#endif /* _SPECVARS_H */
