/* HickUP 1.2 -- Graphical interface to hick.
** Copyright (C) 1998 - 2001, 2008 Daniel Bergstrom <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <string.h>

#include <gtk/gtkmenu.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkhbox.h>
#include <gtk/gtkentry.h>
#include <gtk/gtktable.h>
#include <gtk/gtkframe.h>
#include <gtk/gtkpixmap.h>
#include <gtk/gtkhpaned.h>
#include <gtk/gtkvpaned.h>
#include <gtk/gtknotebook.h>
#include <gtk/gtkmenuitem.h>
#include <gtk/gtkspinbutton.h>
#include <gtk/gtkoptionmenu.h>
#include <gtk/gtkcheckbutton.h>

#include "gui.h"
#include "wwrap.h"
#include "menupop.h"
#include "buttons.h"


static const char *_menu_match[MENU_MATCH_SIZE] = {
  "None",
  "Match",
  "Dosen't Match",
  "Case Match",
  "Dosen't Case Match"
#ifdef USE_REGEX
  ,"Regex Match",
  "Dosen't Regex Match"
#endif /* USE_REGEX */
};

static const char *_selection[SELECTION_SIZE] = {
  "Hidden",
  "Not Selected",
  "Selected",
  "Default",
  "Forced",
  "Required"
};

static const char *_menu_date[MENU_DATE_SIZE] = {
  "All",
  "Created After",
  "Created Before",
  "Modified After",
  "Modified Before",
  "Expires After",
  "Expires Before"
};


static GtkWidget *create_ptab(const char *name, TabPackets *ptab, 
                              GtkWidget *notebook, GUI_Pixmaps *guip, 
                              Project *prj);

static void cb_expand_edit(GtkCTree *ctree, GtkCTreeNode *node, gpointer data);

static GPixmap *tabs_get_pixmap(AvailPacket *apack, AvailVersion *aver, 
                                void *data);

static void tabs_reqchk_init_edit(ReqChk *rchk);
static void tabs_reqchk_init_packets(ReqChk *rchk);


void
create_packets_tab(const char *name, TabPackets *ptab, GtkWidget *notebook, 
                   GUI_Pixmaps *guip, Project *prj)
{
  create_ptab(name, ptab, notebook, guip, prj);

  ptab->is_edit = 0;
  filter_init(&ptab->filter);
  reqchk_init(&ptab->reqchk, &ptab->avail, prj, tabs_reqchk_init_packets,
              reqchk_check_packets);
  reqchk_set_button_ok(&ptab->reqchk, cb_apply, prj);
  reqchk_set_button_ignore(&ptab->reqchk, cb_update, prj);
  reqchk_set_button_cancel(&ptab->reqchk, cb_reqchk_cancel, &ptab->reqchk);
  reqchk_set_title(&ptab->reqchk, "Requied Packets");
  reqchk_set_help(&ptab->reqchk, "The following packets have not been "
                  "selected. To get the packets in the list working "
                  "correctly,will you need to select them.");

  hview_menu_create(&ptab->hview,
                    "Select",  &_gmain.pixmaps.selected, MENU_ENABLE, 
                    cb_pack_select,
                    "Select Defaults", &_gmain.pixmaps.dselected,
                    MENU_DEFAULT, cb_pack_select_default,
                    "Unselect", &_gmain.pixmaps.select, MENU_DISABLE,
                    cb_pack_unselect,
                    "Unselect Packet", &_gmain.pixmaps.select,
                    MENU_DISABLE_PACKET, cb_pack_unselect_packet, NULL);

  gtk_signal_connect(GTK_OBJECT(ptab->hview.view), "tree_select_row", 
                     GTK_SIGNAL_FUNC(cb_select), ptab);
  gtk_signal_connect(GTK_OBJECT(ptab->hview.view), "tree_expand", 
                     GTK_SIGNAL_FUNC(cb_expand_edit), (gpointer)2);
  gtk_signal_connect(GTK_OBJECT(ptab->hview.view), "tree_collapse", 
                     GTK_SIGNAL_FUNC(cb_expand_edit), (gpointer)3);
  gtk_signal_connect(GTK_OBJECT(ptab->hview.view), "key_press_event", 
                     GTK_SIGNAL_FUNC(cb_pack_key_item), &ptab->hview);
  gtk_signal_connect(GTK_OBJECT(ptab->hview.view), "button_press_event", 
                     GTK_SIGNAL_FUNC(cb_pack_click_item), &ptab->hview);

  /* Required Pakcets Popup Menu */

  hview_menu_create(&ptab->require,
                    "View", NULL, MENU_VIEW, cb_pack_view,
                    "", NULL, 0, NULL,
                    "Select",  &_gmain.pixmaps.selected, MENU_ENABLE,
                    cb_pack_select,
                    "Select Defaults",   &_gmain.pixmaps.dselected,
                    MENU_DEFAULT, cb_pack_select_default,
                    "Unselect", &_gmain.pixmaps.select, MENU_DISABLE, 
                    cb_pack_unselect,
                    "Unselect Packet",   &_gmain.pixmaps.select,
                    MENU_DISABLE_PACKET, cb_pack_unselect_packet, NULL);

  gtk_signal_connect(GTK_OBJECT(ptab->require.view), "tree_select_row", 
                     GTK_SIGNAL_FUNC(cb_select_ctree), &ptab->require);
  gtk_signal_connect(GTK_OBJECT(ptab->require.view), "key_press_event", 
                     GTK_SIGNAL_FUNC(cb_pack_key_item), &ptab->require);
  gtk_signal_connect(GTK_OBJECT(ptab->require.view), "button_press_event", 
                     GTK_SIGNAL_FUNC(cb_pack_click_item), &ptab->require);

  /* Conflicting Pakcets Popup Menu */

  hview_menu_create(&ptab->conflict,
                    "View", NULL, MENU_VIEW, cb_pack_view,
                    "", NULL, 0, NULL,
                    "Select",  &_gmain.pixmaps.selected, MENU_ENABLE,
                    cb_pack_select,
                    "Select Defaults",   &_gmain.pixmaps.dselected,
                    MENU_DEFAULT, cb_pack_select_default,
                    "Unselect", &_gmain.pixmaps.select, MENU_DISABLE, 
                    cb_pack_unselect,
                    "Unselect Packet",   &_gmain.pixmaps.select,
                    MENU_DISABLE_PACKET, cb_pack_unselect_packet, NULL);

  gtk_signal_connect(GTK_OBJECT(ptab->conflict.view), "select_row", 
                     GTK_SIGNAL_FUNC(cb_select_clist), &ptab->conflict);
  gtk_signal_connect(GTK_OBJECT(ptab->conflict.view), "key_press_event", 
                     GTK_SIGNAL_FUNC(cb_pack_key_item), &ptab->conflict);
  gtk_signal_connect(GTK_OBJECT(ptab->conflict.view), "button_press_event", 
                     GTK_SIGNAL_FUNC(cb_pack_click_item), &ptab->conflict);
}


void
create_edit_tab(const char *name, TabPackets *ptab, GtkWidget *notebook, 
                GUI_Pixmaps *guip, Project *prj)
{
  create_ptab(name, ptab, notebook, guip, prj);

  ptab->is_edit = 1;
  ptab->copy_from = 0;

  filter_init(&ptab->filter);
  reqchk_init(&ptab->reqchk, &ptab->avail, prj, tabs_reqchk_init_edit,
              reqchk_check_edit);
  reqchk_set_button_ok(&ptab->reqchk, cb_edit_apply, prj);
  reqchk_set_button_ignore(&ptab->reqchk, cb_edit_save, prj);
  reqchk_set_button_cancel(&ptab->reqchk, cb_reqchk_cancel, &ptab->reqchk);
  reqchk_set_title(&ptab->reqchk, "Required Packets");
  reqchk_set_help(&ptab->reqchk, "The following packets have not been "
                  "enabled. To allow the users in the project to select so "
                  "that the programs will work you need to enable them.");

  hview_pixmap_set_func(&ptab->require, reqchk_pixmap);
  hview_pixmap_set_func(&ptab->conflict, reqchk_pixmap);

  /* Packet Tree Popup Menu */

  hview_menu_create(&ptab->hview,
                    "Copy from project...", NULL, MENU_COPY_PRJ,
                    cb_edit_copy_prj,
                    "", NULL, 0, NULL,
                    "Enable",  &_gmain.pixmaps.selected, MENU_ENABLE,  
                    cb_edit_enable,
                    "Enable Defaults",   &_gmain.pixmaps.selected,
                    MENU_DEFAULT, cb_edit_enable_default,
                    "Enable Packet",   &_gmain.pixmaps.dselected,
                    MENU_ENABLE_PACKET, cb_edit_enable_packet,
                    "Disable", &_gmain.pixmaps.select, MENU_DISABLE, 
                    cb_edit_disable,
                    "Disable Packet",   &_gmain.pixmaps.select,
                    MENU_DISABLE_PACKET, cb_edit_disable_packet,
                    "Force",   &_gmain.pixmaps.forced, MENU_FORCE,   
                    cb_edit_force, NULL);

  gtk_signal_connect(GTK_OBJECT(ptab->hview.view), "tree_select_row", 
                     GTK_SIGNAL_FUNC(cb_select), ptab);
  gtk_signal_connect(GTK_OBJECT(ptab->hview.view), "tree_expand", 
                     GTK_SIGNAL_FUNC(cb_expand_edit), (gpointer)0);
  gtk_signal_connect(GTK_OBJECT(ptab->hview.view), "tree_collapse", 
                     GTK_SIGNAL_FUNC(cb_expand_edit), (gpointer)1);
  gtk_signal_connect(GTK_OBJECT(ptab->hview.view), "key_press_event", 
                     GTK_SIGNAL_FUNC(cb_edit_key_item), &ptab->hview);
  gtk_signal_connect(GTK_OBJECT(ptab->hview.view), "button_press_event", 
                     GTK_SIGNAL_FUNC(cb_edit_click_item), &ptab->hview);

  /* Required Pakcets Popup Menu */

  hview_menu_create(&ptab->require,
                    "View", NULL, MENU_VIEW, cb_edit_view,
                    "", NULL, 0, NULL,
                    "Enable", &_gmain.pixmaps.selected, MENU_ENABLE, 
                    cb_edit_enable,
                    "Enable Defaults",   &_gmain.pixmaps.selected,
                    MENU_DEFAULT, cb_edit_enable_default,
                    "Enable Packet",   &_gmain.pixmaps.dselected,
                    MENU_ENABLE_PACKET, cb_edit_enable_packet,
                    "Disable", &_gmain.pixmaps.select, MENU_DISABLE, 
                    cb_edit_disable,
                    "Disable Packet",   &_gmain.pixmaps.select,
                    MENU_DISABLE_PACKET, cb_edit_disable_packet,
                    "Force", &_gmain.pixmaps.forced, MENU_FORCE, 
                    cb_edit_force, NULL);

  gtk_signal_connect(GTK_OBJECT(ptab->require.view), "tree_select_row", 
                     GTK_SIGNAL_FUNC(cb_select_ctree), &ptab->require);
  gtk_signal_connect(GTK_OBJECT(ptab->require.view), "key_press_event", 
                     GTK_SIGNAL_FUNC(cb_edit_key_item), &ptab->require);
  gtk_signal_connect(GTK_OBJECT(ptab->require.view), "button_press_event", 
                     GTK_SIGNAL_FUNC(cb_edit_click_item), &ptab->require);

  /* Conflicting Pakcets Popup Menu */

  hview_menu_create(&ptab->conflict,
                    "View", NULL, MENU_VIEW, cb_edit_view,
                    "", NULL, 0, NULL,
                    "Enable", &_gmain.pixmaps.selected, MENU_ENABLE, 
                    cb_edit_enable,
                    "Enable Defaults",   &_gmain.pixmaps.selected,
                    MENU_DEFAULT, cb_edit_enable_default,
                    "Enable Packet",   &_gmain.pixmaps.dselected,
                    MENU_ENABLE_PACKET, cb_edit_enable_packet,
                    "Disable", &_gmain.pixmaps.select, MENU_DISABLE, 
                    cb_edit_disable,
                    "Disable Packet",   &_gmain.pixmaps.select,
                    MENU_DISABLE_PACKET, cb_edit_disable_packet,
                    "Force", &_gmain.pixmaps.forced, MENU_FORCE, 
                    cb_edit_force, NULL);

  gtk_signal_connect(GTK_OBJECT(ptab->conflict.view), "select_row", 
                     GTK_SIGNAL_FUNC(cb_select_clist), &ptab->conflict);
  gtk_signal_connect(GTK_OBJECT(ptab->conflict.view), "key_press_event", 
                     GTK_SIGNAL_FUNC(cb_edit_key_item), &ptab->conflict);
  gtk_signal_connect(GTK_OBJECT(ptab->conflict.view), "button_press_event", 
                     GTK_SIGNAL_FUNC(cb_edit_click_item), &ptab->conflict);
}


static void
cb_expand_edit(GtkCTree *ctree, GtkCTreeNode *ctnode, gpointer data)
{
  GPixmap *pixi;
  NodeTree *ntree;
  RefPacket *pref;

  /* data is for expand / collaps, edit: 0..1, packets: 2..3 */

  if(!(ntree = (NodeTree *)gtk_ctree_node_get_row_data(ctree, ctnode)) ||
     (ntree->node.flag != NT_PACKET)) {
    return;
  }

  pref = (RefPacket *)ntree;

  if(pref->packet->info.install != INSTALL_NOT) {
    if((data <= (gpointer)1) && 
       (pref->packet->info.install == INSTALL_DEFAULT)) {
      return;
    }

    if((data == (gpointer)0) || (data == (gpointer)2)) {
      gtk_ctree_node_set_pixmap(ctree, ctnode, 0, _gmain.pixmaps.select.pix,
                                _gmain.pixmaps.select.mask);
    } else {
      pixi = tabs_get_pixmap(pref->packet, NULL, NULL);
      gtk_ctree_node_set_pixmap(ctree, ctnode, 0, pixi->pix, pixi->mask);
    }
  }
}


static GtkWidget *
create_filter_menu(const char **array, size_t siz, GtkWidget *menuitems[])
{
  size_t n;
  GtkWidget *menu;

  menu = gtk_menu_new();

  for(n = 0; n < siz; n++) {
    menuitems[n] = gtk_menu_item_new_with_label(array[n]);
    gtk_menu_append(GTK_MENU(menu), menuitems[n]);
    gtk_widget_show(menuitems[n]);
  }

  return menu;
}


void
create_filter_tab(TabFilter *ftab, GtkWidget *notebook)
{
  size_t n;
  GtkAdjustment *adj;
  GtkWidget *vbox, *frame, *hbox, *fbox, *fvbox, *w;

  filter_init(&ftab->filter);

  vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
  w = gtk_label_new("Filter");
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, w);

  /* String Match */

  frame = gtk_frame_new("String Match");
  gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 0);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
  gtk_container_add(GTK_CONTAINER(frame), hbox);

  w = create_filter_menu(_menu_match, MENU_MATCH_SIZE, ftab->match_menus);
  ftab->match = gtk_option_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(ftab->match), w);
  gtk_box_pack_start(GTK_BOX(hbox), ftab->match, FALSE, TRUE, 0);

  w = gtk_label_new("String");
  gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, TRUE, 5);

  ftab->string = gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(hbox), ftab->string, TRUE, TRUE, 0);

  /* Date */

  frame = gtk_frame_new("Date");
  gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, TRUE, 0);

  hbox = gtk_hbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
  gtk_container_add(GTK_CONTAINER(frame), hbox);

  w = create_filter_menu(_menu_date, MENU_DATE_SIZE, ftab->date_menus);
  ftab->dateset = gtk_option_menu_new();
  gtk_option_menu_set_menu(GTK_OPTION_MENU(ftab->dateset), w);
  gtk_box_pack_start(GTK_BOX(hbox), ftab->dateset, FALSE, TRUE, 0);

  w = gtk_label_new("Date");
  gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, TRUE, 0);

  adj = GTK_ADJUSTMENT(gtk_adjustment_new(1.0, 1.0, 31.0, 1.0, 5.0, 0.0));

  ftab->day = gtk_spin_button_new(adj, 0, 0);
  gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(ftab->day), TRUE);
  gtk_box_pack_start(GTK_BOX(hbox), ftab->day, FALSE, FALSE, 0);

  adj = GTK_ADJUSTMENT(gtk_adjustment_new(1.0, 1.0, 12.0, 1.0, 5.0, 0.0));

  ftab->month = gtk_spin_button_new(adj, 0, 0);
  gtk_signal_connect(GTK_OBJECT(ftab->month), "changed",
                     GTK_SIGNAL_FUNC(cb_filter_update_date), ftab);
  gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(ftab->month), TRUE);
  gtk_box_pack_start(GTK_BOX(hbox), ftab->month, FALSE, FALSE, 0);

  adj = GTK_ADJUSTMENT(gtk_adjustment_new(0.0, 0.0, 9999.0, 1.0, 100.0,0.0));

  ftab->year = gtk_spin_button_new(adj, 0, 0);
  gtk_signal_connect(GTK_OBJECT(ftab->year), "changed",
                     GTK_SIGNAL_FUNC(cb_filter_update_date), ftab);
  gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(ftab->year), FALSE);
  gtk_widget_set_usize(ftab->year, 55, -1);
  gtk_box_pack_start(GTK_BOX(hbox), ftab->year, FALSE, FALSE, 0);

  /* Show Selected */ 

  fbox = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox), fbox, FALSE, TRUE, 0);

  frame = gtk_frame_new("Hide Selection");
  gtk_box_pack_start(GTK_BOX(fbox), frame, TRUE, TRUE, 0);

  fvbox = gtk_vbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(fvbox), 5);
  gtk_container_add(GTK_CONTAINER(frame), fvbox);

  for(n = 0; n < SELECTION_SIZE; n++) {
    ftab->select_hide[n] =  gtk_check_button_new_with_label(_selection[n]);
    gtk_box_pack_start(GTK_BOX(fvbox), ftab->select_hide[n], FALSE, TRUE, 0);
  }

  frame = gtk_frame_new("Show Selection");
  gtk_box_pack_start(GTK_BOX(fbox), frame, TRUE, TRUE, 0);

  fvbox = gtk_vbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(fvbox), 5);
  gtk_container_add(GTK_CONTAINER(frame), fvbox);

  for(n = 0; n < SELECTION_SIZE; n++) {
    ftab->select_show[n] =  gtk_check_button_new_with_label(_selection[n]);
    gtk_box_pack_start(GTK_BOX(fvbox), ftab->select_show[n], FALSE, TRUE, 0);
  }

  /* Buttons */

  ww_buttons(vbox, WW_HBBOX, GTK_BUTTONBOX_START, 0, 
             "New Packets", BUTTON_NODEF, NULL, cb_filter_newpackets, ftab, 
             "Reset", BUTTON_NODEF, NULL, cb_filter_reset, ftab,
             NULL);

  gtk_widget_show_all(vbox);
}


static GtkWidget *
create_ptab(const char *name, TabPackets *ptab, GtkWidget *notebook, 
            GUI_Pixmaps *guip, Project *prj)
{
  gint cwidth[2];
  gchar *titles[2];
  GtkWidget *mpaned, *hpaned, *vpaned, *w, *hbox, *vbox;

  ptab->ctnode = NULL;
  ptab->has_init = 0;

  hsh_nodelist_init(&ptab->avail);

  cwidth[0] = 16;
  cwidth[1] = -1;
  titles[0] = "";

  mpaned = gtk_hpaned_new();
  gtk_container_set_border_width(GTK_CONTAINER(mpaned), 5);
  gtk_paned_set_gutter_size(GTK_PANED(mpaned), 14);
  w = gtk_label_new(name);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), mpaned, w);

  titles[1] = "Packets";
  w = hview_new(&ptab->hview, HVIEW_CTREE, 2, titles, cwidth, prj);
  hview_avail_set(&ptab->hview, &ptab->avail);
  hview_pixmap_set_func(&ptab->hview, tabs_get_pixmap);
  gtk_clist_column_titles_hide(hview_clist_get(&ptab->hview));

  gtk_widget_set_usize(hview_widget_get(&ptab->hview), 190, -1);
  gtk_paned_pack1(GTK_PANED(mpaned), w, FALSE, TRUE);

  vbox = gtk_vbox_new(FALSE, 5);
  gtk_paned_pack2(GTK_PANED(mpaned), vbox, TRUE, TRUE);

  hbox = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);

  ptab->pixmap = gtk_pixmap_new(guip->pixmaps[0]->close.pix, 
                                guip->pixmaps[0]->close.mask);
  gtk_box_pack_start(GTK_BOX(hbox), ptab->pixmap, FALSE, TRUE, 0);

  ptab->packet = gtk_label_new("");
  gtk_box_pack_start(GTK_BOX(hbox), ptab->packet, FALSE, TRUE, 0);

  ptab->version = gtk_label_new("");
  gtk_box_pack_start(GTK_BOX(hbox), ptab->version, FALSE, TRUE, 0);

  ptab->info = gtk_label_new("");
  gtk_box_pack_start(GTK_BOX(hbox), ptab->info, FALSE, TRUE, 0);

  vpaned = gtk_vpaned_new();
  gtk_box_pack_start(GTK_BOX(vbox), vpaned, TRUE, TRUE, 0);
  gtk_paned_set_gutter_size(GTK_PANED(vpaned), 10);

  w = ww_text_area(&ptab->about, 0);
  gtk_paned_pack1(GTK_PANED(vpaned), w, FALSE, TRUE);

  hpaned = gtk_hpaned_new();
  gtk_paned_set_gutter_size(GTK_PANED(hpaned), 10);
  gtk_widget_set_usize(hpaned, -1, 103);
  gtk_paned_pack2(GTK_PANED(vpaned), hpaned, FALSE, TRUE);

  titles[1] = "Requires";
  w = hview_new(&ptab->require, HVIEW_CTREE, 2, titles, cwidth, prj);
  gtk_paned_pack1(GTK_PANED(hpaned), w, TRUE, TRUE);

  titles[1] = "Conflicts";
  w = hview_new(&ptab->conflict, HVIEW_CLIST, 2, titles, cwidth, prj);
  gtk_paned_pack2(GTK_PANED(hpaned), w, TRUE, TRUE);

  gtk_widget_show_all(mpaned);
  gtk_widget_hide(ptab->pixmap);

  hview_avail_set(&ptab->require, &ptab->avail);
  hview_pixmap_set_func(&ptab->require, select_pixmap_require);

  hview_avail_set(&ptab->conflict, &ptab->avail);
  hview_pixmap_set_func(&ptab->conflict, select_pixmap_conflict);
  return vbox;
}


static GPixmap *
tabs_get_pixmap(AvailPacket *apack, AvailVersion *aver, void *data)
{
  if(apack) {
    if(!aver && data && (apack->info.install != INSTALL_DEFAULT)) {
      return &_gmain.pixmaps.select;
    }

    switch((aver ? aver->info.install : apack->info.install)) {
      case INSTALL_NOT: {
        return &_gmain.pixmaps.select;
      }
      case INSTALL_DEFAULT: {
        return &_gmain.pixmaps.dselected;
      }
      case INSTALL_FORCE: {
        return &_gmain.pixmaps.forced;
      }
      case INSTALL_VERSION: {
        return &_gmain.pixmaps.selected;
      }
    }
  }

  return &_gmain.pixmaps.attention;
}


static void
tabs_reqchk_init_edit(ReqChk *rchk)
{
  hview_menu_create(&rchk->packets,
                    "View", NULL, MENU_VIEW, cb_edit_view,
                    "", NULL, 0, NULL,
                    "Enable", &_gmain.pixmaps.selected, MENU_ENABLE, 
                    cb_edit_enable,
                    "Enable Defaults",   &_gmain.pixmaps.selected,
                    MENU_DEFAULT, cb_edit_enable_default,
                    "Enable Packet",   &_gmain.pixmaps.dselected,
                    MENU_ENABLE_PACKET, cb_edit_enable_packet,
                    "Disable", &_gmain.pixmaps.select, MENU_DISABLE, 
                    cb_edit_disable,
                    "Disable Packet",   &_gmain.pixmaps.select,
                    MENU_DISABLE_PACKET, cb_edit_disable_packet,
                    "Force", &_gmain.pixmaps.forced, MENU_FORCE, 
                    cb_edit_force, NULL);

  gtk_signal_connect(GTK_OBJECT(rchk->packets.view), "select_row", 
                     GTK_SIGNAL_FUNC(cb_select_clist), &rchk->packets);
  gtk_signal_connect(GTK_OBJECT(rchk->packets.view), "key_press_event", 
                     GTK_SIGNAL_FUNC(cb_edit_key_item), &rchk->packets);
  gtk_signal_connect(GTK_OBJECT(rchk->packets.view), "button_press_event", 
                     GTK_SIGNAL_FUNC(cb_edit_click_item), &rchk->packets);

  hview_menu_create(&rchk->require,
                    "View", NULL, MENU_VIEW, cb_edit_view,
                    "", NULL, 0, NULL,
                    "Enable", &_gmain.pixmaps.selected, MENU_ENABLE, 
                    cb_edit_enable,
                    "Enable Defaults",   &_gmain.pixmaps.selected,
                    MENU_DEFAULT, cb_edit_enable_default,
                    "Enable Packet",   &_gmain.pixmaps.dselected,
                    MENU_ENABLE_PACKET, cb_edit_enable_packet,
                    "Disable", &_gmain.pixmaps.select, MENU_DISABLE, 
                    cb_edit_disable,
                    "Disable Packet",   &_gmain.pixmaps.select,
                    MENU_DISABLE_PACKET, cb_edit_disable_packet,
                    "Force", &_gmain.pixmaps.forced, MENU_FORCE, 
                    cb_edit_force, NULL);

  gtk_signal_connect(GTK_OBJECT(rchk->require.view), "tree_select_row", 
                     GTK_SIGNAL_FUNC(cb_select_ctree), &rchk->require);
  gtk_signal_connect(GTK_OBJECT(rchk->require.view), "key_press_event", 
                     GTK_SIGNAL_FUNC(cb_edit_key_item), &rchk->require);
  gtk_signal_connect(GTK_OBJECT(rchk->require.view), "button_press_event", 
                     GTK_SIGNAL_FUNC(cb_edit_click_item), &rchk->require);
}


static void
tabs_reqchk_init_packets(ReqChk *rchk)
{
  hview_menu_create(&rchk->packets,
                    "View", NULL, MENU_VIEW, cb_pack_view,
                    "", NULL, 0, NULL,
                    "Select",  &_gmain.pixmaps.selected, MENU_ENABLE,
                    cb_pack_select,
                    "Select Defaults", &_gmain.pixmaps.dselected,
                    MENU_DEFAULT, cb_pack_select_default,
                    "Unselect", &_gmain.pixmaps.select, MENU_DISABLE, 
                    cb_pack_unselect,
                    "Unselect Packet", &_gmain.pixmaps.select,
                    MENU_DISABLE_PACKET, cb_pack_unselect_packet,
                    NULL);

  gtk_signal_connect(GTK_OBJECT(rchk->packets.view), "select_row", 
                     GTK_SIGNAL_FUNC(cb_select_clist), &rchk->packets);
  gtk_signal_connect(GTK_OBJECT(rchk->packets.view), "key_press_event", 
                     GTK_SIGNAL_FUNC(cb_pack_key_item), &rchk->packets);
  gtk_signal_connect(GTK_OBJECT(rchk->packets.view), "button_press_event", 
                     GTK_SIGNAL_FUNC(cb_pack_click_item), &rchk->packets);

  hview_menu_create(&rchk->require,
                    "View", NULL, MENU_VIEW, cb_pack_view,
                    "", NULL, 0, NULL,
                    "Select",  &_gmain.pixmaps.selected, MENU_ENABLE,
                    cb_pack_select,
                    "Select Defaults",   &_gmain.pixmaps.dselected,
                    MENU_DEFAULT, cb_pack_select_default,
                    "Unselect", &_gmain.pixmaps.select, MENU_DISABLE, 
                    cb_pack_unselect,
                    "Unselect Packet",   &_gmain.pixmaps.select,
                    MENU_DISABLE_PACKET, cb_pack_unselect_packet,
                    NULL);

  gtk_signal_connect(GTK_OBJECT(rchk->require.view), "tree_select_row", 
                     GTK_SIGNAL_FUNC(cb_select_ctree), &rchk->require);
  gtk_signal_connect(GTK_OBJECT(rchk->require.view), "key_press_event", 
                     GTK_SIGNAL_FUNC(cb_pack_key_item), &rchk->require);
  gtk_signal_connect(GTK_OBJECT(rchk->require.view), "button_press_event", 
                     GTK_SIGNAL_FUNC(cb_pack_click_item), &rchk->require);
}
