/* HickUP 1.2 -- Graphical interface to hick.
** Copyright (C) 1998 - 2001, 2008 Daniel Bergstrom <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef _TABS_H
#define _TABS_H

typedef struct  _tabfilter TabFilter;

#include "hview.h"
#include "refs.h"
#include "array.h"
#include "reqchk.h"
#include "filter.h"

/* TabPackets ************************************************************** */

typedef struct {
  HView hview;

  GtkWidget *pixmap;
  GtkWidget *packet;
  GtkWidget *version;
  GtkWidget *info;

  GtkWidget *about;

  HView require;
  HView conflict;

  Filter filter;             /* filter for avail packets */
  NodeList avail;            /* AvailPackets */

  unsigned int is_edit;      /* is the edit page? */
  unsigned int has_init : 2; /* has it been initialized? 
                             ** 0 - not initialized, 1 - initialized,
                             ** 2 - reinitialize 
                             */

  GtkCTreeNode *ctnode;

  char *old_s1;
  char *old_s2;

  ReqChk reqchk;
  void *copy_from;          /* copy from project, internal data. */
} TabPackets;

/* TabFilter *************************************************************** */

#define SELECTION_SIZE   6
#define MENU_DATE_SIZE   7

#ifndef USE_REGEX
#define MENU_MATCH_SIZE  5
#else  /* USE_REGEX */
#define MENU_MATCH_SIZE  7
#endif /* USE_REGEX */


struct _tabfilter {
  GtkWidget *match; /* what kind of match type */
  GtkWidget *match_menus[MENU_MATCH_SIZE];
  GtkWidget *string;

  GtkWidget *dateset; /* how to lookup the date */
  GtkWidget *date_menus[MENU_DATE_SIZE];
  GtkWidget *day;
  GtkWidget *month;
  GtkWidget *year;

  GtkWidget *select_show[SELECTION_SIZE]; /* selection to show */
  GtkWidget *select_hide[SELECTION_SIZE]; /* selection to hide */

  Filter filter;
};


#ifdef __cplusplus
extern "C" {
#endif

extern void create_packets_tab(const char *name, TabPackets *ptab, 
                               GtkWidget *notebook, GUI_Pixmaps *guip,
                               Project *prj);
extern void create_edit_tab(const char *name, TabPackets *ptab, 
                            GtkWidget *notebook, GUI_Pixmaps *guip,
                            Project *prj);

extern void create_filter_tab(TabFilter *ftab, GtkWidget *notebook);

/* in select.c */
extern void cb_select(GtkCTree *ctree, GtkCTreeNode *ctnode, gint column, 
                      TabPackets *ptab);

extern void select_clear(TabPackets *ptab);

extern GPixmap *select_pixmap_conflict(AvailPacket *apack, AvailVersion *aver,
                                       void *data);
extern GPixmap *select_pixmap_require(AvailPacket *apack, AvailVersion *aver, 
                                      void *data);

#ifdef __cplusplus
}
#endif

#endif /* _TABS_H */
