/* HList 1.0.0 -- Packet information extractor.
** Copyright (C) 2000 Daniel Bergstroem <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>

#include "hick.h"

#define PROGRAM_NAME     "hlist"
#define PROGRAM_VERSION  "1.0.0"

static int
usage(const char *filename)
{
  fprintf(stderr, "Usage: %s [ OPTIONS ] PACKET_PATH HICKUPTYPE\n\n"
          "\t-h\t\tThis help text.\n"
          "\t-v\t\tVersion information.\n\n", filename);
  return 0;
}

int
main(int argc, char **argv)
{
  int n;
  HickPackets packets;
  HickEntry entry;
  char *packet_path = argv[1];
  char *hickuptype = argv[2];

  while((n = getopt(argc, argv, "hvd:")) != EOF) {
    switch(n) {
      case 'v': {               /* version information */
        printf(PROGRAM_NAME " " PROGRAM_VERSION "\n");
        return 0;
      }
      case 'h':                 /* usage info */
      default: {                /* ERROR! */
        return usage(argv[0]);
      }
    }
  }

  if(argc != 3) {
    return usage(argv[0]);
  }

  switch((n = openPackets(packet_path, "/", hickuptype, 0, &packets))) {
    case 0: {
      break;
    }
    case HICKERR_PACKDIR: {
      fprintf(stderr, "Failed to open packet directory '%s'.\n", packet_path);
      return 0;
    }
    case HICKERR_OUTOFMEMORY: {
      fprintf(stderr, "hlist: Out of memory -- exiting.\n");
      return 0;
    }
    default: {
      fprintf(stderr, "hlist: unknown packet error (%d).\n", n);
      return 0;
    }
  }

  initHickEntry(&entry);

  while(nextPacket(&packets, &entry, 0)) {
    printf("%s\n", entry.he_name);
  }

  closePackets(&packets);

  return 0;
}
