/* HMerge 1.0.3 -- Packet merger.
** Copyright (C) 2000 Daniel Bergstroem <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <ctype.h>
#include "hmerge.h"
#include "version.h"

static void hmerge_merge_versions(const char *number, HickPacket *hpac);

void
hmerge_unite_versions(NodeList *list)
{
  Node *node;
  char *number;
  HMergePacket *mergepkg, *mpkg;

  if(!list->head || !list->head->next) {
    return;
  }

  mergepkg = (HMergePacket *)list->head;

  if(!mergepkg->pkg.pac_versions.vl_head) {
    return;
  }

  number = mergepkg->pkg.pac_versions.vl_head->ver_number;

  hmerge_merge_versions(number, &mergepkg->pkg);

  for(node = list->head->next; node; node = node->next) {
    mpkg = (HMergePacket *)node;

    hmerge_merge_versions(number, &mpkg->pkg);
  }

}

void
hmerge_merge_versions(const char *number, HickPacket *hpac)
{
  HickVer *ver;

  for(ver = hpac->pac_versions.vl_head; ver; ver = ver->ver_next) {
    strcpy(ver->ver_number, number);
  }

  mergeHickPacket(hpac, 0);
}

int
hmerge_merge_packets(HickPacket *pkg, NodeList *packets)
{
  Node *node;
  HMergePacket *mpkg;
  HickVer *ver, *nver;

  for(node = packets->head; node; node = node->next) {
    mpkg = ((HMergePacket *)node);

    for(ver = mpkg->pkg.pac_versions.vl_head; ver; ver = nver) {
      nver = ver->ver_next;

      if(_versionFind(ver->ver_number, &pkg->pac_versions)) {
        fprintf(stderr, "duplicate copies of version '%s'\n", ver->ver_number);
        continue;
      }

      _versionAppend(ver, &pkg->pac_versions);
    }
  }

  return 0;
}

int
hmerge_load_file(HMergePacket *mpkg)
{
  FILE *file;
  HickEntry entry;

  if(!(file = fopen(mpkg->filename, "r"))) {
    fprintf(stderr, "Failed to open '%s'.\n", mpkg->filename);
    return 0;
  }

  initHickEntry(&entry);
  
  if(!_getPacketHead(file, &entry)) {
    freeHickEntry(&entry);
    fclose(file);
    fprintf(stderr, "Failed to load '%s': packet header error.\n", 
            mpkg->filename);
    return 0;
  }

  strcpy(mpkg->pkg.pac_name, entry.he_name);

  mpkg->pkg.pac_groups.arr = entry.he_groups.arr;
  mpkg->pkg.pac_groups.siz = entry.he_groups.siz;
  mpkg->pkg.pac_groups.alloc = entry.he_groups.alloc;

  hsh_array_init(&entry.he_groups);

  _getPacket(file, 0, &mpkg->pkg, 1);
  fclose(file);
  return 1;
}

int
hmerge_get_weight(char *filename) 
{
  int n;
  char *ptr;

  for(n = strlen(filename); (n >= 0) && (filename[n] != '-'); n--);

  if(n < 0) {
    return -1;
  }

  ptr = filename + n + 1;

  while(filename[++n]) {
    if(!isdigit(filename[n])) {
      return -1;
    }
  }

  return atoi(ptr);
}
