/* HMerge 1.0.3 -- Packet merger.
** Copyright (C) 2000 Daniel Bergstroem <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "hmerge.h"

#include <stdio.h>

static int
usage(const char *filename)
{
  fprintf(stderr, "Usage: %s [ OPTIONS ] FILE [ FILE ... ]\n\n"
          "\t-h\t\tThis help text.\n"
          "\t-v\t\tVersion information.\n"
          "\t-d VERSION\tSpecifies default version(s). The option can "
          "be used\n\t\t\tmultiple times, If no default set first version "
          "will be \n\t\t\tset as the default.\n"
          "\t-O\t\tMerge all versions into one, first packets first\n"
          "\t\t\tversion will be the one merged to.\n\n", filename);
  return 0;
}


int
main(int argc, char **argv)
{
  int n, weight;
  NodeList list;
  HMergePacket *mpkg;
  PArr defvers;
  char *deffile = 0;
  int oneversion = 0;
  int priority_diff = 0;

  parr_init(&defvers);

  while((n = getopt(argc, argv, "hvd:OD:p:")) != EOF) {
    switch(n) {
      case 'v': { /* version information */
        printf(PROGRAM_NAME " " PROGRAM_VERSION "\n");
        return 0;
      }
      case 'd': { /* default version */
        parr_add(&defvers, optarg);
        break;
      }
      case 'O': { /* one version */
        oneversion = 1;
        break;
      }
      case 'D': { /* file containing default values */
        deffile = optarg;
        break;
      }
      case 'p': { /* priority change */
        priority_diff = atoi(optarg);
        break;
      }
      case 'h':   /* usage info */
      default: {  /* ERROR! */
        return usage(argv[0]);
      }
    }
  }

  if(deffile) {
    fprintf(stderr, "default file: %s\n", deffile);
  }

  if(optind >= argc) {
    return usage(argv[0]);
  }

  hsh_nodelist_init(&list);

  for(n = optind; n < argc; n++) {
    if((weight = hmerge_get_weight(argv[n])) == -1) {
      fprintf(stderr, "error: not a valid filename '%s'.\n", argv[n]);
      continue;
    }
      
    if(!(mpkg = MallocMem(HMergePacket))) {
      fprintf(stderr, "error: Out of memory!\n");
      return 1;
    }

    mpkg->num.num = weight;
    mpkg->filename = argv[n];
    initHickPacket(&mpkg->pkg);

    if(!hmerge_load_file(mpkg)) {
      FreeMem(mpkg);
      continue;
    }

    if(oneversion) {
      hsh_node_append(&mpkg->num.node, &list);
    } else {
      hsh_node_num_insert(&mpkg->num, &list, 0);
    }
  }

  if(!list.head) {
    fprintf(stderr, "error: no packets loaded -- exiting.\n");
    return 1;
  }

  if(oneversion) {
    hmerge_unite_versions(&list);
  }

  mpkg = ((HMergePacket *)list.head);
  hsh_node_remove(list.head, &list);

  if(!hmerge_merge_packets(&mpkg->pkg, &list)) {
    if(oneversion) {
      mergeHickPacket(&mpkg->pkg, 0);
    }

    hmerge_default_set(&mpkg->pkg, &defvers);

    savePacket(stdout, &mpkg->pkg, 0, HTIME_DATE);
  }

  return 0;
}
