/* HMod 1.0.1 -- Hick modules emulation.
** Copyright (C) 2000 - 2001 Daniel Bergstrom <daniel@octocode.com>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2, or (at your option)
** any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software Foundation,
** Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "hmod.h"
#include "hmodlist.h"
#include "hmodavail.h"

#ifdef HAVE_FNMATCH
#include <fnmatch.h>
#else
#undef FNM_PATHNAME /* some sytems got this defined in their unistd.h */
#define FNM_PATHNAME (1 << 0)
#endif

#define SCREEN_WIDTH 80
#define GET_SYMBOL(pkg) (pkg->avail == HMOD_AVAIL_NOT ? '-' : \
                         (pkg->avail == HMOD_AVAIL_LOADED ? '+' : '~'))
#define PACKET_COMPARE(hpkg, node) \
       (strcasecmp(hpkg->name, ((HModPacket *)node)->name) <= 0)


static HModPacket *hmod_avail_packet_insert(HModPacket *hpkg, NodeList *list);


void
hmod_avail_print(FILE *file, HickPackets *pkg, char **match, size_t msiz)
{
  PArr arr;
  Node *node;
  HModPacket *cpkg;
  HModPackets hmodpkg;
  size_t cols, rows, row, col;

  fprintf(file, "In directory %s:\n\n", PACKET_PATH);  

  hmod_list_get(&arr);

  if(!hmod_avail_get(&hmodpkg, pkg, &arr, match, msiz)) {
    return;
  }

  if(hmodpkg.have_def) {
    hmodpkg.width += 8;
  }

  cols = SCREEN_WIDTH / (hmodpkg.width + 3);

  if(cols < 1) {
    cols = 1;
  }

  rows = ((hmodpkg.siz - 1) / cols) + 1;

  for(node = hmodpkg.list.head, row = 0; row < rows && node; row++) {
    for(col = 0; (col < cols) && node; col++, node = node->next) {
      cpkg = (HModPacket *)node;

      if(cpkg->is_def) {
        fprintf(file, "  %c%s/%s%-*s", GET_SYMBOL(cpkg), cpkg->name, 
                cpkg->version, 
                (int)(hmodpkg.width - strlen(cpkg->name) -
                      strlen(cpkg->version) - 1),
                " (def) ");
      } else {
        fprintf(file, "  %c%s/%-*s", GET_SYMBOL(cpkg), cpkg->name, 
                (int)(hmodpkg.width - strlen(cpkg->name) - 1), cpkg->version);
      }
    }

    putc('\n', file);
  }
}


int
hmod_avail_get(HModPackets *hmodpkg, HickPackets *pkg, PArr *loaded,
               char **match, size_t msiz)
{
  HickVer *ver;
  time_t curTime;
  HModPacket *pnew;
  HickEntry entry;
  size_t wname, wver;

  time(&curTime);

  hmodpkg->siz = 0;
  hmodpkg->width = 0;
  hmodpkg->have_def = 0;
  hsh_nodelist_init(&hmodpkg->list);

  initHickEntry(&entry);

  while(nextPacket(pkg, &entry, 0)) {
    wname = strlen(entry.he_name);

    for(ver = entry.he_versions.vl_head; ver; ver = ver->ver_next) {
      if(ver->ver_expire && (ver->ver_expire > curTime)) {
        continue;
      }

      if(msiz) {
        size_t n, ok = 0;

        for(n = 0; n < msiz; n++) {
          if(!strncasecmp(match[n], entry.he_name, strlen(match[n]))) {
            ok = 1;
          }
        }

        if(!ok) {
          continue;
        }
      }

      wver = strlen(ver->ver_number);

      hmod_out_of_memory((pnew = MallocMem(HModPacket)));

      pnew->name    = hmod_strdup(entry.he_name);
      pnew->version = hmod_strdup(ver->ver_number);
      pnew->avail   = HMOD_AVAIL_NOT;
      pnew->is_def  = ver->ver_default;

      if(ver->ver_default) {
        hmodpkg->have_def = 1;
      }

      if(hsh_array_find(ver->ver_number, &entry.he_using)) {
        pnew->avail = HMOD_AVAIL_BASE;
      }

      if(hmod_list_find(loaded, pnew->name, pnew->version)) {
        pnew->avail = HMOD_AVAIL_LOADED;
      }

      hmod_avail_packet_insert(pnew, &hmodpkg->list);

      hmodpkg->siz++;
      hmodpkg->width = ((wname + wver) > hmodpkg->width ? (wname + wver) :
                        hmodpkg->width);
    }
  }

  return 1;
}


static HModPacket *
hmod_avail_packet_insert(HModPacket *hpkg, NodeList *list)
{
  Node *node;

  if(!list->head || PACKET_COMPARE(hpkg, list->head)) {
    return (HModPacket *)hsh_node_prepend(&hpkg->node, list);
  }

  for(node = list->head; node; node = node->next) {
    if(PACKET_COMPARE(hpkg, node)) {
      return (HModPacket *)hsh_node_insert_after(&hpkg->node, node, list);
    }
  }

  return (HModPacket *)hsh_node_append(&hpkg->node, list);
}
