/* HICK Lib 1.3.2 -- User Profiler Library.
** Copyright (C) 1997 - 2000 Daniel Bergstrom <daniel@octocode.com>
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _COMMON_H
#define _COMMON_H

#include "memd.h"
#include "hick.h"

#define TMPBUFSIZ   30
#define MKDIR_MODE  493
#define BUFLEN      64
#define BUFFERTSIZ  1024

#define COMMON_DIR     "common"
#define PACKETS_DIR    "packets"
#define SCRIPTS_DIR    "scripts"

#define INSTALLED_FILE "installed"
#define CHECK_FILE     "chkTime"
#define LOG_FILE       "runlog"
#define CREATED_FILE   "created"
#define ENVIRON_FILE   "environ"    /* env      */
#define INFO_FILE      "info"       /* info     */
#define HTML_FILE      "html"       /* html     */
#define MENUS_FILE     "menus"      /* menuItem */
#define XRES_FILE      "Xresources" /* xres     */
#define CALL_SCRIPT    "call"       /* call     */

#define XRES_HEADER "! This file was generated by HICK 1.3.1\n\n"
#define CALL_HEADER "#!/bin/sh\n# This file was generated by HICK 1.3.1\n\n"

#define IS_DOT_OR_DOTDOT(e) (((e)->d_name[0] == '.') && \
    (((e)->d_name[1] == (char)0) || \
     (((e)->d_name[1] == '.') && ((e)->d_name[2] == (char)0))))

#define IS_EMACS_BACKUP(e, l) (((e)->d_name[l] == '~') || \
       (((e)->d_name[0] == '#') && ((e)->d_name[l] == '#')))

/* Buffert ***************************************************************** */

typedef struct {
  char buf[BUFFERTSIZ+1]; /* string buffert */
  unsigned int pos;       /* current position in buffert */
  FILE *file;             /* file pointer to write to. */
} Buffert;


#define bput(c, b) \
      if(((b)->pos + 1) >= BUFFERTSIZ) { \
        fwrite((b)->buf, (b)->pos, 1, (b)->file); \
        (b)->pos = 0; \
      } \
      (b)->buf[(b)->pos++] = (c)

#define bputs(s, b) _bputs((s), strlen((s)), (b))

#ifdef __cplusplus
extern "C" {
#endif

extern int _convertString(const char *s, char *buf, size_t maxlen);

extern struct dirent *_getNextEntry(HickPackets *pack);

extern void bflush(Buffert *b);
extern void bprint(Buffert *buf, const char *fmt, ...);
extern void _bputs(const char *s, size_t len, Buffert *b);
extern void _convertBstring(const char *s, int c, Buffert *b);

/* defined in: rescue.c */

extern int _unlinkRecursive(const char *dirname, HickEvent *ev,
                            HickEventFunction call, void *arg);

#ifdef __cplusplus
}
#endif

#endif /* _COMMON_H */
