/* HickMK 1.2.2 -- HICK Make Utility.
** Copyright (C) 1998 - 2001 Daniel Bergstrom <daniel@octocode.com>
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "mkparse.h"


static void mkparse_putback(MKInterp *interp, char sign, int is_not, SBuf *var,
                            SBuf *stmt);


char *
mkparse_string(const char *s, MKInterp *interp)
{
  interp->buf.siz = 0;

  mkparse_parse(s, interp);
  sbuf_put((char)0, &interp->buf);

  return interp->buf.buf;
}


char *
mkparse_parse(const char *s, MKInterp *interp)
{
  MKVariable *var;
  size_t n = 0, is_not;
  SBuf vstr = { NULL, 0, 0 }, sstr = { NULL, 0, 0 };

  while(s[n]) {
    if(s[n] == '$') {
      is_not = 0;
      sstr.siz = vstr.siz = 0;

      switch(s[++n]) {
        case '$': { /* $$ -> $ */
          sbuf_put(s[n++], &interp->buf);
          continue;
        }
        case '!': { /* $! -> not expression */
          if(s[++n] != '{') { /* it's not  $!{ -> fail */
            sbuf_put('$', &interp->buf);
            sbuf_put('!', &interp->buf);
            continue;
          }

          is_not = 1; /* continue down */
        }
        case '{': {
          for(n++; s[n] && (s[n] != '}'); n++) {
            sbuf_put(s[n], &vstr);
          }

          sbuf_put((char)0, &vstr);

          if(s[n] != '}') {
            mkparse_putback(interp, '$', is_not, &vstr, NULL);
            continue;
          }

          if(s[++n] == '[') { /* statement */
            for(n++; s[n]; n++) {
              if(s[n] == ']') {
                if(s[n + 1] != ']') {
                  break;
                }

                n++;
              }

              sbuf_put(s[n], &sstr);
            }

            sbuf_put((char)0, &sstr);

            if(s[n] != ']') {
              mkparse_putback(interp, '$', is_not, &vstr, &sstr);
              continue;
            }

            n++;
          }

          var = mkvar_find(interp, vstr.buf);

          if(sstr.siz) { /* got statement */
            if((is_not && (!var || !var->value.buf || !var->value.buf[0])) ||
               (!is_not && var && (var->vfunc ||
                                   (var->value.buf && var->value.buf[0])))) {

              if(var && var->vfunc) {
                if((var->vfunc)(var, interp, is_not, 1)) {
                  mkparse_parse(sstr.buf, interp);
                }
              } else {
                mkparse_parse(sstr.buf, interp);
              }
            }
          } else {
            if(var && (var->vfunc || (var->value.buf && var->value.buf[0]))) {
              if(var->vfunc) {
                (var->vfunc)(var, interp, is_not, 0);
             } else {
                if(!is_not) {
                  sbuf_putsb(&var->value, &interp->buf);
                }
              }
            } else { /* no ID or value */
              if(!var || (var && (!var->value.buf || !var->value.buf[0]))) {
                if(is_not) {
                  sbuf_put('1', &interp->buf);
                }
              }
            }
          }

          continue;
        }
        default: {
          sbuf_put('$', &interp->buf);
        }
      }
    }

    sbuf_put(s[n++], &interp->buf);
  }

  if(vstr.buf) {
    FreeMem(vstr.buf);
  }

  if(sstr.buf) {
    FreeMem(sstr.buf);
  }

  return interp->buf.buf;
}


static void
mkparse_putback(MKInterp *interp, char sign, int is_not, SBuf *var, SBuf *stmt)
{
  sbuf_put(sign, &interp->buf);

  if(is_not) {
    sbuf_put('!', &interp->buf);
  }

  sbuf_put('{', &interp->buf);
  sbuf_putsb(var, &interp->buf);

  if(stmt) {
    sbuf_put('}', &interp->buf);
    sbuf_put('[', &interp->buf);
    sbuf_putsb(stmt, &interp->buf);
  }
}
