/* HickMK 1.2.2 -- HICK Make Utility.
** Copyright (C) 1998 - 2001 Daniel Bergstrom <daniel@octocode.com>
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <sys/types.h>
#include <unistd.h>
#include <pwd.h>

#include "common.h"
#include "utils.h"
#include "getfile.h"


char *
hmk_getlogin(void)
{
  struct passwd *pwd;

  if(!(pwd = getpwuid(geteuid()))) {
    return NULL;
  }

  return pwd->pw_name;
}


char *
_getCString(FILE *file)
{
  int c;
  char *buf;
  unsigned int siz = 0, asiz = REALLOC_SIZE;

  if((c = fgetc(file)) == EOF) {
    return NULL;
  }

  buf = MallocArr(char, asiz);
 
  for( ; (c != EOF) && (c != ':') && (c != '\n'); c = fgetc(file)) {
    if(c == '\\') {
      if((c = fgetc(file)) == EOF) {
        break;
      }

      if((c == ':') || (c == '\\')) {
        _addChar(buf, siz, asiz, c);
        continue;
      }

      if(c == 'n') {
        _addChar(buf, siz, asiz, '\n');
        continue;
      }

      _addChar(buf, siz, asiz, '\\');
    }

    _addChar(buf, siz, asiz, c);
  }

  _addChar(buf, siz, asiz, 0);
  return buf;
}


char **
class_get(const char *path, const char *file, enum MKClass cl, size_t *siz)
{
  size_t n;
  Node *node;
  NodeList funcs;
  char fname[FNAME_LEN+1], **p = NULL;

  hsh_nodelist_init(&funcs);

  hsh_create_str(fname, FNAME_LEN, path, "/", file, NULL);

  if(mklang_functions_load(fname, &funcs)) {
    *siz = 0;
    return NULL;
  }

  for(*siz = 0, node = funcs.head; node; node = node->next) {
    if(((MKFunction *)node)->fclass == cl) {
      (*siz)++;
    }
  }

  if(siz) {
    if(!(p = MallocArr(char *, *siz))) {
      *siz = 0;
      return NULL;
    }

    for(n = 0, node = funcs.head; node; node = node->next) {
      if(((MKFunction *)node)->fclass == cl) {
        p[n++] = ((MKFunction *)node)->name;
        ((MKFunction *)node)->name = NULL;
      }
    }
  }

  mklang_functions_free(&funcs);
  return p;
}
