/* HickSH 1.1.2 -- Shared routines between hick parts.
** Copyright (C) 1999 - 2001 Daniel Bergstrom <daniel@octocode.com>
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <string.h>

#include "memd.h"
#include "array.h"

#define parr_get(parr, type, n)  ((type *)(parr)->arr[n])

/* For all PArr functions:
**
**   returns 0 on failre, any other is the position where the data where added
**   plus one. parr_get() excepted.
**
** The add functions will not copy the information at the pointer but set the
** pointer where it adds it to point at it.
*/

size_t
parr_add(PArr *parr, void *data)
{
  if(parr->siz >= parr->alloc) {
    if(parr->arr) {
      parr->arr = ReMallocArr(parr->arr, void *, parr->alloc + PARR_SIZE);
    } else {
      parr->arr = MallocArr(void *, PARR_SIZE);
    }

    if(!parr->arr) {
      return 0;
    }

    parr->alloc += PARR_SIZE;
  }

  parr->arr[parr->siz++] = data;
  return parr->siz;
}


size_t
parr_find_add(PArr *parr, void *data, PArrFind func)
{
  size_t n;

  n = parr_find(parr, data, func);

  if(n) {
    return n;
  }

  return parr_add(parr, data);
}


void
parr_move(PArr *to, PArr *from)
{
  size_t n;

  if(!to->siz) {
    to->arr = from->arr;
    to->siz = from->siz;
    to->alloc = from->alloc;
  } else {
    for(n = 0; n < from->siz; n++) {
      parr_add(to, from->arr[n]);
    }

    if(from->arr) {
      FreeMem(from->arr);
    }
  }

  parr_init(from);
}


void *
parr_remove(PArr *parr, size_t n)
{
  void *p;
  size_t i;

  if(n >= parr->siz) {
    return NULL;
  }

  p = parr->arr[n];

  for(i = n + 1; i < parr->siz; i++, n++) {
    parr->arr[n] = parr->arr[i];
  }

  parr->siz--;
  return p;
}


void
parr_clear(PArr *parr)
{
  size_t n;

  if(parr->arr) {
    for(n = 0; n < parr->siz; n++) {
      FreeMem(parr->arr[n]);
    }

    FreeMem(parr->arr);
    parr_init(parr);
  }
}


size_t
parr_find(PArr *parr, const void *data, PArrFind func)
{
  size_t n;

  if(func) {
    for(n = 0; n < parr->siz; n++) {
      if(!(func)(parr->arr[n], data)) {
        return n + 1;
      }
    }
  } else {
    for(n = 0; n < parr->siz; n++) {
      if(parr->arr[n] == data) {
        return n + 1;
      }
    }
  }

  return 0;
}


void *
_parr_find_value(PArr *parr, const void *data, PArrFind func)
{
  size_t n;

  if(!(n = parr_find(parr, data, func))) {
    return NULL;
  }

  return parr->arr[n - 1];
}


int
parr_strcasecmp(const void *d1, const void *d2)
{
  return strcasecmp((char *)d1, (char *)d2);
}


int
parr_strcmp(const void *d1, const void *d2)
{
  return strcmp((char *)d1, (char *)d2);
}
