/* HickSH 1.1.2 -- Shared routines between hick parts.
** Copyright (C) 1999 - 2001 Daniel Bergstrom <daniel@octocode.com>
**
** This library is free software; you can redistribute it and/or
** modify it under the terms of the GNU Library General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This library is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** Library General Public License for more details.
**
** You should have received a copy of the GNU Library General Public
** License along with this library; if not, write to the Free
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _ARRAY_H
#define _ARRAY_H

#include <sys/types.h>

#define PARR_SIZE 5

/* PArrFind **************************************************************** */

typedef int (*PArrFind)(const void *d1, const void *d2);

/* PArr ******************************************************************** */

typedef struct {
  void **arr;   /* allocated array with the data */
  size_t siz;   /* number if items in array */
  size_t alloc; /* allocated size */
} PArr;

#ifdef __cplusplus
extern "C" {
#endif

#define parr_init(parr) (parr)->arr = NULL, (parr)->siz = (parr)->alloc = 0

#define parr_get(parr, type, n)  ((type *)(parr)->arr[n])

#define parr_find_value(parr, type, data, func)  \
  ((type *)_parr_find_value((parr), (data), (func)))

extern size_t parr_add(PArr *parr, void *data);
extern size_t parr_find_add(PArr *parr, void *data, PArrFind func);

extern void parr_move(PArr *to, PArr *from);
extern void *parr_remove(PArr *parr, size_t n);
extern void parr_clear(PArr *parr);

extern size_t parr_find(PArr *parr, const void *data, PArrFind func);

extern void *_parr_find_value(PArr *parr, const void *data, PArrFind func);

/* Predefined compare functions */

extern int parr_strcmp(const void *d1, const void *d2);
extern int parr_strcasecmp(const void *d1, const void *d2);

#ifdef __cplusplus
}
#endif


#endif /* _ARRAY_H */
