package Project;

use strict;
use Env qw(USER HICKTYPE PROJECT PROJECT_ROOT SHELL);

my $projectrc = ".projectrc";
my $hickprjrc = ".hick/project-";

sub get_home {
    my $home = (getpwuid($>))[7];
    die "No information found about user #$USER.\n" if(!defined $home);
    return $home;
}

sub get_gid { 
    return ($_[0] eq $USER ? (getpwnam($_[0]))[3] : get_project_gid($_[0]));
}

sub get_name {
    my ($proj, $subproj) = @_;
    return ($subproj ? "$proj:$subproj" : $proj);
}

sub get_dotted_name {
    my ($proj, $subproj) = @_;
    my $name = ($subproj ? "$proj:$subproj" : $proj);
    $name =~ s/:/./g;
    return $name;
}

sub get_escaped_name {
    my ($proj, $subproj) = @_;
    my $name = ($subproj ? "$proj:$subproj" : $proj);
    $name =~ s/:/\\\\:/g;
    return $name;
}

sub get_project {
    (my $proj = $_[0]) =~ s/^([^:]*).*/$1/;
    my $subproj;
    $subproj = $1 if $_[0] =~ /^[^:]*:(.*)$/;
    $proj = $USER unless $proj;
    $proj =~ s/\+$//;
    $subproj = "" unless $subproj;
    return $proj, $subproj;
}

sub get_current {
    return ($PROJECT ? get_project($PROJECT) : ());
}

sub is_member {
    my ($proj, $subproj) = @_;
    my @groups = map { my $x = getgrgid($_); $x =~ s/\+$//; $x } sort numeric split(' ', $();
    my @mainprojs;
    my $projects;

    return 1 if($proj eq $USER);

    push @mainprojs, get_projuser($USER);
    push @mainprojs, get_projgroup(@groups);
    $projects = get_projects();

    die "Main project '$proj' not found.\n" unless defined $projects->{$proj};

    foreach (@mainprojs) {
	return 1 if $_ eq $proj;
    }

    die "Your not a member of project '$proj'.\n";
    return 0;
}

sub enter {
    my ($project_root, $proj, $subproj) = @_;
    $PROJECT_ROOT = $project_root;
    $PROJECT = get_name($proj, $subproj);
    my $gid = get_project_gid($proj, $subproj);

    chdir($PROJECT_ROOT) || die "error: Can't chdir to '$PROJECT_ROOT': !$\n";
    print "============== Login to project $PROJECT ================\n";

    if($PROJECT eq $USER) {
	$gid = (getpwuid($>))[3];
	die "Can't lookup user information for #$>.\n" unless defined $gid;
    }

    if (defined $gid) {
        my $group = (getgrgid($gid))[0];
        die "Can't lookup group name for #$gid.\n" unless defined $group;
        exec "newgrp", $group;
    }
    exec $SHELL;
}

# Used by project -n
sub init_project {
    my ($gid, $proj, $subproj) = @_;
    my $project_root = get_project_root($proj);
    
    if( ! -d $project_root) {
	mkdir($project_root, 0775) || die "$0: Failed to create '$project_root': $!\n";
	chown $>, $gid, $project_root if defined $gid;
    }
    if( ! -d "$project_root/.hick") {
	mkdir("$project_root/.hick", 0775) || die "$0: Failed to create '$project_root/.hick': $!\n";
	chown $>, $gid, "$project_root/.hick" if defined $gid;
    }
    my $site = get_site();
    if( ! -d "$project_root/.hick/$site") {
	mkdir("$project_root/.hick/$site", 0775) || die "$0: Failed to create '$project_root/.hick/$site': $!\n";
	chown $>, $gid, "$project_root/.hick/$site" if defined $gid;
    }
    if( ! -d "$project_root/.hick/$site/" . get_dotted_name($proj, $subproj)) {
	mkdir("$project_root/.hick/$site/" . get_dotted_name($proj, $subproj), 0775) ||
            die "$0: Failed to create '$project_root/.hick/$site/" . get_dotted_name($proj, $subproj) . "': $!\n";
	chown $>, $gid, "$project_root/.hick/$site/" . get_dotted_name($proj, $subproj) if defined $gid;
    }
    create_file($gid, "$project_root/.hick/$site/" . get_dotted_name($proj, $subproj) . "/prjinfo.hick");
    create_file($gid, "$project_root/.hick/$site/" . get_dotted_name($proj, $subproj) . "/prjspec.hick");
}

sub create_file {
    my ($gid, $file) = @_;

    if ( ! -r $file && open(FILE, ">$file")) {
	close(FILE);
	chmod(0644, $file);
	chown $>, $gid, $file if defined $gid;
	return 1;
    }

    return 0;
}

sub get_sub_projects {
    my ($project, $dir) = @_;
    my @result = ();

    opendir(DIR, "$dir") || return @result;
    my @files = grep /^$project\./, readdir DIR;
    closedir(DIR);

    foreach(@files) {
        (my $project = $_) =~ s/\./:/g;
        push @result, [ get_project($project) ] if -d "$dir/$_";
    }

    return @result;
}

sub get_projuser {
    my ($user) = @_;
    my (@maingroups);

    if ( -r "/etc/projuser" ) {
        if (open(FILE, "/etc/projuser")) {
            while (<FILE>) {
                chomp;
		next if /^#/;
                push @maingroups, split /:+/, $1 if /^$user:(.*)$/;
            }
            close FILE;
        }
    }
    return @maingroups;
}

sub get_projgroup {
    my (@maingroups);

    if ( -r "/etc/projgroup" ) {
        if (open(FILE, "/etc/projgroup")) {
            while (<FILE>) {
                chomp;
		next if /^#/;
                foreach my $group (@_) {
                    push @maingroups, split /:+/, $1 if /^$group:(.*)$/;
                }
            }
            close FILE;
        }
    }
    return @maingroups;
}

sub get_projects {
    my %projects;

    if ( -r "/etc/projects" ) {
        if (open(FILE, "/etc/projects")) {
            while (<FILE>) {
                chomp;
		next if /^#/;
                $projects{$1} = $2 if /^([^:]+):(.+)/;
            }
            close FILE;
        }
    }
    return \%projects;
}

sub get_project_root {
    my ($proj) = @_;

    return get_home() if $proj eq $USER;

    if ( -r "/etc/projects" ) {
        if (open(FILE, "/etc/projects")) {
            while (<FILE>) {
                chomp;
		next if /^#/;
                if (/^${proj}:([^:]*)/) {
                    close FILE;
                    return $1;
                }
            }
            close FILE;
        }
    }
    return "";
}

sub get_project_gid {
    my ($group) = @_;

    if ( -r "/etc/projects" ) {
        if (open(FILE, "/etc/projects")) {
            while (<FILE>) {
                chomp;
		next if /^#/;
                if (/^$group:[^:]*:([^:]+)/) {
                    close FILE;
                    return $1;
                }
            }
            close FILE;
        }
    }
    return undef;
}

sub get_site {
    if ( -r "@basepath@/etc/site" ) {
	if (open(FILE, "@basepath@/etc/site")) {
            chomp(my $site = <FILE>);
            close FILE;
            return $site;
        }
    }
    return "undef";
}
            
sub get {
    my ($group_name, $projects);
    my (@maingroups, @result);
    my @groups = map { my $x = getgrgid($_); $x =~ s/\+$//; $x } sort numeric split(' ', $();
    my $site = get_site();

    push @maingroups, get_projuser $USER;
    push @maingroups, get_projgroup @groups;
    push @result, get_sub_projects($USER, get_home() . "/.hick/$site");
    $projects = get_projects;

    if ( defined $projects ) {
        foreach (@maingroups) {
            next unless exists $projects->{$_};
            my ($project_root, $gid) = split /:+/, $projects->{$_};

            push @result, [$_, ""];
            push @result, get_sub_projects($_, "$project_root/.hick/$site");
        }
    }

    return @result;
}

sub numeric { $a <=> $b }

# Get info about all projects in one : separated string. (used by hickup and hickprj)
sub gets {
    my ($parameter) = @_;
    my @groups = Project::get();
    my $result;

    return "" unless @groups;

    my $site = get_site();
    foreach(@groups) {
	my ($proj, $subproj) = @{$_};
	$result .= ':' if $result;
	$result .= ($subproj ? get_escaped_name($proj, $subproj) : $proj);
	$result .= ":" . get_project_root($proj) . "/.hick/$site/" . get_dotted_name($proj, $subproj);
	$result .= ":" . get_project_root($proj) . "/.hick/$site/" . get_dotted_name($proj, $subproj) . "/$USER";
    }
    $result = "$parameter $result" if $parameter;

    return $result;
}

sub update_projectrc {
    my $home = get_home();

    if(open(DEFPRJ, "$home/$hickprjrc$HICKTYPE")) {
        my $prjstr = <DEFPRJ>;
        chomp $prjstr;
        close(DEFPRJ);

        my ($proj, $subproj) = get_project($prjstr);

        if ($proj ne $USER && ! $subproj) {
            write_projectrc($home, $proj, $subproj,
                          Project::get_project_root($proj));
            return;
        }
    }

    unlink "$home/$projectrc";
}

sub write_hickprjrc {
    my ($home, $proj, $subproj) = @_;

    open(PROJRC, ">$home/$hickprjrc$HICKTYPE") ||
	die "$0; Unable to open '$home/$hickprjrc$HICKTYPE' for write: $!\n";
    print PROJRC "$proj" . ($subproj ? ":$subproj" : "") . "\n";
    close PROJRC;
}

sub write_projectrc {
    my ($home, $proj, $subproj, $project_root) = @_;

    open(PROJRC, ">$home/$projectrc") ||
        die "$0: Unable to open '$home/$projectrc' for write: $!\n";

    my $prjstr = $proj . ($subproj ? ":$subproj" : "");

    print PROJRC "setenv PROJECT	$prjstr\n";
    print PROJRC "setenv PROJECT_ROOT	$project_root\n";
    close PROJRC;
}

sub remove_hickprjrc {
    my ($home) = @_;

    unlink("$home/$hickprjrc$HICKTYPE") || 
	print "$0: Unable to unlink '$home/$hickprjrc$HICKTYPE': $!\n"
	    if( -f "$home/$hickprjrc$HICKTYPE" );
}

sub remove_projectrc {
    my ($home) = @_;

    unlink("$home/$projectrc") || 
        print "$0: Unable to unlink '$home/$projectrc': $!\n"
            if( -f "$home/$projectrc" );
}

1;
