#!@PERL@ -w
use strict;
use lib '@prefix@/lib';
use Getopt::Long;
use hick::Types;
use hick::Project;
use Env qw(USER HICKUPROOT HICKUPTYPE LD_LIBRARY_PATH);

my ($path, $prog, $suff) = $0 =~ m,(.*/)?([^./]*)(.*)?$,;
my $hickbin = "@bindir@";
my $nutsbin = "@nuts@ -w -t \"HickUP Information\" -n -";
my $usage = <<END;
Usage: $prog [ OPTIONS... ] [ HICKUPTYPE ] [ HICKUPROOT ]
        -h, --help              Prints this help text.
        -r, --rescue            Checks for the need to run rescue.
        -u, --user              Updates the current user hick setup.
        -f, --full              Updates the current user and all projects.
        -U PROJECT,
        --update=PROJECT        Updates the project named.
        -q                      Decrease verbosity.
END

use vars qw/ $opt_h $opt_r $opt_u $opt_f $opt_U $opt_q /;
my %opts = ("h"        => \$opt_h, "help"     => \$opt_h,
	    "r"        => \$opt_r, "rescue"   => \$opt_r,
	    "u"        => \$opt_u, "user"     => \$opt_u,
	    "f"        => \$opt_f, "full"     => \$opt_f,
            "U=s"      => \$opt_U, "update=s" => \$opt_U,
	    "q"        => \$opt_q, "quiet"    => \$opt_q
	    );

Getopt::Long::Configure("no_ignore_case");
GetOptions(%opts) || die $usage;

die $usage if($opt_h);

die "Error: Can't use -r with -U, -u or -f.\n"
    if($opt_r && ($opt_u || $opt_f || $opt_U));

die "Error: Can't mix -U, -u and -f.\n"
    if(($opt_U && ($opt_u || $opt_f)) || ($opt_u && $opt_f));

my $hick_opts = "";
$hick_opts = "-r" if($opt_r);
$hick_opts = "-u" if($opt_u);
$hick_opts = "-f" if($opt_f);
$hick_opts = "-U \"$opt_U\"" if(defined $opt_U);

$LD_LIBRARY_PATH = "@GTKLIBS@" . 
    (defined $LD_LIBRARY_PATH ? ":$LD_LIBRARY_PATH" : "");

die $usage if(!Types::check(@ARGV) || ($#ARGV != -1));

my $prjstr = Project::gets("-p");

my $userhickuproot = `$hickbin/get-userhickuproot`;
chomp $userhickuproot;

system("$hickbin/hickup.bin $hick_opts -P $userhickuproot " . $prjstr . 
        ($opt_q ? " > /dev/null 2>&1" : ""));
