#!/bin/sh

echo=echo
[ "`echo -e`" != "-e" ] && echo='echo -e'

if [ "$HICKTYPE" = "" ]; then
    HICKTYPE=`@bindir@/get-hicktype`

    if [ $HICKTYPE = "" ]; then
	$echo "Failed to get HICKTYPE - aborting."
	exit 0;
    fi

    export HICKTYPE
    $echo "No HICKTYPE set, using '$HICKTYPE'."
fi

HICKHPKG=@datadir@/hpkgs/$HICKTYPE
HICKPKGS=@datadir@/hick/packets/$HICKTYPE

create_dir() {
    if [ ! -d "$1" ]; then
	$echo "Creating directory ($1)."
	mkdir -m755 "$1"
    fi
}

create_dir "@datadir@/hpkgs"
create_dir $HICKHPKG
create_dir "@datadir@/hick"
create_dir "@datadir@/hick/packets"

run_visp() {
    case $visp_pos in
	"\\") $echo "|\b\c";     visp_pos="|";;
	"|")  $echo "/\b\c";     visp_pos="/";;
	"-")  $echo "\\\\\b\c";  visp_pos="\\";;
	*)    $echo "-\b\c";     visp_pos="-";;
    esac
}

create_packets() {
    rm -f /tmp/packets.$$

    $echo "Generating packets file... \c"
    for i in $HICKHPKG/*.hpkg ; do
	if [ -f $i ]; then
	    run_visp
	    cat $i >> /tmp/packets.$$
	fi
    done

    $echo "done."
    if [ -f /tmp/packets.$$ ]; then
	create_dir $HICKPKGS
	mv /tmp/packets.$$ $HICKPKGS/packets
	chmod 644 $HICKPKGS/packets
    fi
}

merge_dir() {
    rm -f $HICKHPKG/$2.hpkg

    if [ ! -x @bindir@/hmerge ]; then
	$echo "Can't find '@bindir@/bin/hmerge'."
    else
	@bindir@/hmerge $1/$2* > $HICKHPKG/$2.hpkg
    fi

    [ ! -z "$DIR" ] && rm -f $HICKHPKG/$2.hpkg
}

merge_all_dirs() {
    $echo "Merging packets... \c"
    for i in `find @basepath@ -type d -a -name pkgs -print` ; do
	if [ -d $i ]; then
	    run_visp
	    merge_dir $i `$echo "$i" | awk -F/ '{ print $4 }'`
	fi
    done
    $echo "done."
}

case "$1" in
    packets)
	create_packets
	exit;;
    remerge) 
	merge_all_dirs
	create_packets
	exit;;
    merge)
	if [ "$2" != "" ] && [ "$3" != "" ]; then
	    if [ -d "$2" ]; then
		merge_dir "$2" "$3"
	    else
		rm -f $HICKHPKG/$3.hpkg
	    fi

	    create_packets
	    exit 0
	fi
esac

$echo "Usage: $0 remerge | packets"
$echo "       $0 merge PKGS-PATH NAME"
$echo
$echo "  packets          Recreates the packets file."
$echo "  remerge          Remerge all packets."
$echo "  merge            Merges in the named packet directory."
$echo
