#!@PERL@ -w

use strict;
use lib '@prefix@/lib';
use Getopt::Long;
use hick::Types;
use hick::Project;
use Env qw(HICKROOT HICKTYPE LD_LIBRARY_PATH USER PROJECT PROJECT_ROOT);


my ($path, $prog, $suff) = $0 =~ m,(.*/)?([^./]*)(.*)?$,;
my $usage = <<END;
Usage: $prog [ -d ] project-name [ : sub-project-name ]
       $prog -n project-name [ : sub-project-name ]
       $prog -c
       $prog -l
       $prog -z
END


use vars qw/ $opt_h $opt_d $opt_c $opt_l $opt_z $opt_n /;
my %opts = (default => \$opt_d,
	    current => \$opt_c,
	    list    => \$opt_l,
	    zero    => \$opt_z,
	    new     => \$opt_n,
	    help    => \$opt_h
	    );

GetOptions(%opts) || die $usage;

die $usage if($opt_h);

if($opt_z) {
    my $home = Project::get_home();
    Project::remove_projectrc($home);
    Project::remove_hickprjrc($home);
    exit;
}

if($opt_l) {
    my @project;
    my @groups = Project::get();

    print "Availible projects:\n\n";

    foreach(@groups) {
	@project = ($_->[0], $_->[1]);
	print "  " . Project::get_name(@project) . "\n";
    }

    print "\n";
    exit;
}

if($opt_c) {
    my @current = Project::get_current();

    if(@current) {
	print "Current project is " . Project::get_name(@current) . ".\n";
    } else {
	print "No current project.\n";
    }
    exit;
}

die $usage if($#ARGV != 0);

exit if(!Types::check_HICKTYPE());

my ($proj, $subproj) = Project::get_project($ARGV[0]);
my $project_root = Project::get_project_root($proj, $subproj);

if(Project::is_member($proj)) {
    if($opt_n) {
	my $gid = Project::get_gid($proj);
        Project::init_project($gid, $proj, $subproj);
	exit;
    }

    die "$0: Failed to find project root for " . Project::get_name($proj, $subproj) .
	".\n" if( ! -d $project_root );

    if($opt_d) {
	my $home = Project::get_home();
        Project::write_projectrc($home, $proj, $subproj, $project_root);
        Project::write_hickprjrc($home, $proj, $subproj);
	exit;
    }

    Project::enter($project_root, $proj, $subproj);
}
